/***************************************** (C) COPYRIGHT 2010 Intersil *****************************************
* File Name         : algo_h264_driver.c
* Author            : tang.sc@intersil.com
* Revision          : 1.2.1.1275
* Date              : 2012-03-29
* Description       : this file focus on h264 sps&pps%slice_head encode, nal encap frame. we encap all frame from
*                       driver base NAL format, just audio&jpeg&mv used reserved NAL type.
* Change log        : 
*                   
****************************************************************************************************************
* THE PRESENT SOFTWARE IS DISTRIBUTED IN THE HOPE THAT IT WILL PROVIDE CUSTOMERS 
* WITH CODING INFORMATION TO SPEED UP THEIR PRODUCT DEVELOPMENT, BUT WITHOUT
* ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS
* FOR A PARTICULAR PURPOSE. AS A RESULT, TECHWELL SHALL NOT BE HELD LIABLE FOR ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
****************************************************************************************************************/
#include	<isil5864/isil_common.h>


/****************************************************************************************************************
* Function Name :   h264_gen_sps_rbsp
* Description   :   process sps encode.
* Input         :   param1 is data ptr, 
*                   param2 is sps's param,
*                   param3 is rbsp len,
* Output        :   null
* Return        :   real encode len
****************************************************************************************************************/
static int  h264_gen_sps_rbsp(void *p_data, h264_sps_t *sps, int rbsp_len)
{
    bs_t    bs, *s;

    s = &bs;
    bs_init(s, p_data, rbsp_len);
    bs_write(s, 8, sps->i_profile_idc);
    bs_write(s, 8, ((sps->b_constraint_set0<<7)|(sps->b_constraint_set1<<6)|(sps->b_constraint_set2<<5)));
    bs_write(s, 8, sps->i_level_idc );
    bs_write_ue(s, sps->i_id);
    bs_write_ue(s, sps->i_log2_max_frame_num - 4);
    bs_write_ue(s, sps->i_poc_type);
    if(sps->i_poc_type == 0)
        bs_write_ue(s, sps->i_log2_max_poc_lsb - 4);
    else if (sps->i_poc_type == 1) {
        int i;
        bs_write(s, 1, sps->b_delta_pic_order_always_zero);
        bs_write_se(s, sps->i_offset_for_non_ref_pic);
        bs_write_se(s, sps->i_offset_for_top_to_bottom_field);
        bs_write_ue(s, sps->i_num_ref_frames_in_poc_cycle);
        for(i = 0; i < sps->i_num_ref_frames_in_poc_cycle; i++)
            bs_write_se(s, sps->i_offset_for_ref_frame[i]);
    }
    bs_write_ue(s, sps->i_num_ref_frames);
    bs_write(s, 1, sps->b_gaps_in_frame_num_value_allowed);
    bs_write_ue(s, sps->i_mb_width - 1);
    bs_write_ue(s, sps->i_mb_height - 1);
    bs_write(s, 1, sps->b_frame_mbs_only);
    if( !sps->b_frame_mbs_only)
        bs_write(s, 1, sps->b_mb_adaptive_frame_field);
    bs_write(s, 1, sps->b_direct8x8_inference);
    bs_write(s, 1, 0);
    bs_write(s, 1, 0);
    bs_rbsp_trailing(s);
    return bs_len(s);
}

/****************************************************************************************************************
* Function Name :   gen_h264_sps
* Description   :   process sps encode's root entrance.
* Input         :   param1 is h264 logic chan's control management struct
*                   param2 is curr frame ptr of this chan
* Output        :   null
* Return        :   real encode len
****************************************************************************************************************/
int  gen_h264_sps(isil_h264_encode_control_t *h264_encode_control, isil_video_frame_tcb_t *frame)
{
    h264_nal_t              *nal;
    h264_nal_bitstream_t    *h264_nal_sps_head;
    H264_NAL_INFO           *nal_info;
    isil_video_packet_tcb_queue_t	*video_packet_queue;
    isil_video_packet_tcb_t	*packet;
    int rbsp_len;

    nal = &frame->nal;
    video_packet_queue = &frame->video_packet_queue;
    packet = video_packet_queue->curr_producer;
    frame->frame_len = 0;
    nal->sps_payload = packet->data;
    rbsp_len = SLICE_HEAD_BUF_SIZE;
    memset(nal->sps_payload, 0, rbsp_len);
    h264_nal_sps_head = (h264_nal_bitstream_t*)nal->sps_payload;
    nal_info = &h264_nal_sps_head->nal;
    nal_info->zero0 = 0;
    nal_info->zero1 = 0;
    nal_info->zero2 = 0;
    nal_info->constant_value_1 = 1;
    nal_info->nal_ref_idc = nal->i_ref_idc = NAL_PRIORITY_HIGHEST;
    nal_info->nal_unit_type = nal->i_type = NAL_SPS;
    nal_info->forbidden_zero_bit = 0;
    rbsp_len -= sizeof(h264_nal_bitstream_t);
    nal->sps_paysize = sizeof(h264_nal_bitstream_t);
    nal->sps_paysize += h264_gen_sps_rbsp(h264_nal_sps_head->rbsp, h264_encode_control->sps, rbsp_len);
    frame->frame_len += nal->sps_paysize;
    return nal->sps_paysize;
}

/****************************************************************************************************************
* Function Name :   h264_gen_pps_rbsp
* Description   :   process pps encode.
* Input         :   param1 is data ptr, 
*                   param2 is sps's param,
*                   param3 is rbsp len,
* Output        :   null
* Return        :   real encode len
****************************************************************************************************************/
static int  h264_gen_pps_rbsp(void *p_data, h264_pps_t *pps, int rbsp_len)
{
    bs_t    bs, *s;

    s = &bs;
    bs_init(s, p_data, rbsp_len);
    bs_write_ue(s, pps->i_id);
    bs_write_ue(s, pps->i_sps_id);
    bs_write(s, 1, pps->b_cabac);
    bs_write(s, 1, pps->b_pic_order);
    bs_write_ue(s, pps->i_num_slice_groups - 1);
    bs_write_ue(s, pps->i_num_ref_idx_l0_active - 1);
    bs_write_ue(s, pps->i_num_ref_idx_l1_active - 1);
    bs_write(s, 1, pps->b_weighted_pred);
    bs_write(s, 2, pps->b_weighted_bipred);
    bs_write_se(s, pps->i_pic_init_qp - 26);
    bs_write_se(s, pps->i_pic_init_qs - 26);
    bs_write_se(s, pps->i_chroma_qp_index_offset);
    bs_write(s, 1, pps->b_deblocking_filter_control);
    bs_write(s, 1, pps->b_constrained_intra_pred);
    bs_write(s, 1, pps->b_redundant_pic_cnt);
    bs_rbsp_trailing(s);
    return bs_len(s);
}

/****************************************************************************************************************
* Function Name :   gen_h264_pps
* Description   :   process pps encode's root entrance.
* Input         :   param1 is h264 logic chan's control management struct
*                   param2 is curr frame ptr of this chan
* Output        :   null
* Return        :   real encode len
****************************************************************************************************************/
int gen_h264_pps(isil_h264_encode_control_t *h264_encode_control, isil_video_frame_tcb_t *frame)
{
    h264_nal_t              *nal;
    h264_nal_bitstream_t    *h264_nal_pps_head;
    H264_NAL_INFO           *nal_info;
    isil_video_packet_tcb_queue_t	*video_packet_queue;
    isil_video_packet_tcb_t	*packet;
    int rbsp_len;

    nal = &frame->nal;
    video_packet_queue = &frame->video_packet_queue;
    packet = video_packet_queue->curr_producer;
    nal->pps_payload = &packet->data[frame->frame_len];
    rbsp_len = (SLICE_HEAD_BUF_SIZE-frame->frame_len);
    h264_nal_pps_head = (h264_nal_bitstream_t*)nal->pps_payload;
    nal_info = &h264_nal_pps_head->nal;
    nal_info->zero0 = 0;
    nal_info->zero1 = 0;
    nal_info->zero2 = 0;
    nal_info->constant_value_1 = 1;
    nal_info->nal_ref_idc = nal->i_ref_idc = NAL_PRIORITY_HIGHEST;
    nal_info->nal_unit_type = nal->i_type = NAL_PPS;
    nal_info->forbidden_zero_bit = 0;
    rbsp_len -= sizeof(h264_nal_bitstream_t);
    nal->pps_paysize = sizeof(h264_nal_bitstream_t);
    nal->pps_paysize += h264_gen_pps_rbsp(h264_nal_pps_head->rbsp, h264_encode_control->pps, rbsp_len);
    frame->frame_len += nal->pps_paysize;
    return nal->pps_paysize;
}

/****************************************************************************************************************
* Function Name :   h264_gen_slice_head
* Description   :   process pps encode.
* Input         :   param1 is data ptr, 
*                   param2 is sps's param,
*                   param3 is rbsp len,
* Output        :   null
* Return        :   real encode len
****************************************************************************************************************/
static int  h264_gen_slice_head(void *p_data, h264_slice_header_t *slice_head, h264_nal_t *nal, int rbsp_len)
{
    bs_t    bs, *s;

    s = &bs;
    bs_init(s, p_data, rbsp_len);
    bs_write_ue(s, slice_head->i_first_mb);
    bs_write_ue(s, slice_head->i_type);
    bs_write_ue(s, slice_head->i_pps_id);
    bs_write(s, slice_head->sps->i_log2_max_frame_num, slice_head->i_frame_num);
    if(nal->i_type == NAL_SLICE_IDR)
        bs_write_ue(s, slice_head->i_idr_pic_id);
    if(slice_head->sps->i_poc_type == 0) {
        slice_head->i_poc_lsb = (slice_head->i_frame_num<<1);
        bs_write(s, slice_head->sps->i_log2_max_poc_lsb, slice_head->i_poc_lsb);
    }
    if(slice_head->i_type == SLICE_TYPE_P || slice_head->i_type == SLICE_TYPE_SP || slice_head->i_type == SLICE_TYPE_B ) {
        bs_write1(s, slice_head->b_num_ref_idx_override);
        if(slice_head->b_num_ref_idx_override==1) {
            if (slice_head->i_type == SLICE_TYPE_P)
                bs_write_ue(s, slice_head->i_frame_num-1);
        }
    }

    /* ref pic list reordering */
    if (slice_head->i_type != SLICE_TYPE_I && slice_head->i_type != SLICE_TYPE_SI) {
        slice_head->b_ref_pic_list_reordering_l0 = 0;
        bs_write1(s, slice_head->b_ref_pic_list_reordering_l0);
    }
    if (nal->i_ref_idc != NAL_PRIORITY_DISPOSABLE) {
        if (nal->i_type == NAL_SLICE_IDR) {
            bs_write1(s, 0);  /* no output of prior pics flag */
            bs_write1(s, 0);  /* long term reference flag */
        } else
            bs_write1(s, 0);  /* adaptive_ref_pic_marking_mode_flag */
    }
    bs_write_se(s, slice_head->i_qp_delta );

    if (s->i_left != 8){
        nal->i_temp_bitalign_content = ((s->p[0])<<s->i_left);
        nal->i_temp_bitalign_number = 8-s->i_left;
    } else {
        nal->i_temp_bitalign_content = 0;
        nal->i_temp_bitalign_number = 0;
    }

    return bs_len(s);
}

/****************************************************************************************************************
* Function Name :   gen_h264_slicehead
* Description   :   process slice head encode's root entrance.
* Input         :   param1 is h264 logic chan's control management struct
*                   param2 is curr frame ptr of this chan
* Output        :   null
* Return        :   real encode len
****************************************************************************************************************/
int gen_h264_slicehead(isil_h264_encode_control_t *h264_encode_control, isil_video_frame_tcb_t *frame)
{
    h264_nal_t              *nal;
    h264_nal_bitstream_t    *h264_nal_slice_head;
    H264_NAL_INFO           *nal_info;
    isil_video_packet_tcb_queue_t	*video_packet_queue;
    isil_video_packet_tcb_t	*packet;
    int rbsp_len, frame_type;
#if defined(PCI_PLATFORM)

#else
    u8  *ch;
    u32 temp_value;
#endif

    nal = &frame->nal;
	frame_type = frame->frame_type;
    video_packet_queue = &frame->video_packet_queue;
    packet = video_packet_queue->curr_producer;
	if(frame_type != H264_FRAME_TYPE_IDR){
		nal->sps_payload = nal->pps_payload = packet->data;
		nal->sps_paysize = nal->pps_paysize = frame->frame_len = 0;
		memset(packet->data, 0, SLICE_HEAD_BUF_SIZE);
	}
    nal->slice_payload = &packet->data[frame->frame_len];
    rbsp_len = (SLICE_HEAD_BUF_SIZE-frame->frame_len);
    h264_nal_slice_head = (h264_nal_bitstream_t*)nal->slice_payload;
    nal_info = &h264_nal_slice_head->nal;
    switch(frame_type){
        case H264_FRAME_TYPE_P:
            nal_info->nal_unit_type = nal->i_type = NAL_SLICE;
            break;
        case H264_FRAME_TYPE_I:
            nal_info->nal_unit_type = nal->i_type = NAL_SLICE;
            break;
        default:
            nal_info->nal_unit_type = nal->i_type = NAL_SLICE_IDR;
            break;            
    }
    nal_info->zero0 = 0;
    nal_info->zero1 = 0;
    nal_info->zero2 = 0;
    nal_info->constant_value_1 = 1;
    nal_info->forbidden_zero_bit = 0;
    nal_info->nal_ref_idc = nal->i_ref_idc = NAL_PRIORITY_LOW;
    rbsp_len -= sizeof(h264_nal_bitstream_t);
    nal->slice_paysize = sizeof(h264_nal_bitstream_t);
    nal->slice_paysize += h264_gen_slice_head(h264_nal_slice_head->rbsp, h264_encode_control->slice_head, nal, rbsp_len);
    frame->frame_len += nal->slice_paysize;
#if defined(PCI_PLATFORM)

#else
    ch =  &packet->data[frame->frame_len];
    switch(frame->frame_len&0x1) {
        case 0:
            nal->i_temp_bitalign_content <<= 8;
            break;
        default:
            frame->frame_len--;
            ch--;
            temp_value = *ch;
            temp_value <<= 8;
            temp_value |= (nal->i_temp_bitalign_content);
            nal->i_temp_bitalign_content = temp_value;
            nal->i_temp_bitalign_number += 8;
            break;
    }
#endif
    frame->i_init_qp = h264_encode_control->pps->i_pic_init_qp;
    return nal->slice_paysize;
}

/****************************************************************************************************************
* Function Name :   gen_ext_nal_audio_header
* Description   :   process audio data encap.
* Input         :   param1 is buf
*                   param2 is audio type
*                   param3 is buf len
* Output        :   null
* Return        :   real encode len
****************************************************************************************************************/
int gen_ext_nal_audio_header(char *buf, int audio_type, int len)
{
    ext_h264_nal_bitstream_t    *ext_nal_audio_head = (ext_h264_nal_bitstream_t*)buf;
    H264_NAL_INFO               *nal_info;
    int ret ;

    nal_info = &ext_nal_audio_head->nal;
    switch(audio_type){
        default:
        case ISIL_AUDIO_PCM:
            ext_nal_audio_head->ext_type = NAL_TYPE_EX_A_PCM;
            break;
        case ISIL_AUDIO_ALAW:
            ext_nal_audio_head->ext_type = NAL_TYPE_EX_A_ALAW;
            break;
        case ISIL_AUDIO_ULAW:
            ext_nal_audio_head->ext_type = NAL_TYPE_EX_A_ULAW;
            break;
        case ISIL_AUDIO_ADPCM_32K:
            ext_nal_audio_head->ext_type = NAL_TYPE_EX_A_IMAADPCM;
            break;
        case ISIL_AUDIO_ADPCM_48K:
            ext_nal_audio_head->ext_type = NAL_TYPE_EX_A_IMAADPCM;
            break;
        case ISIL_AUDIO_ADPCM_16K:
            ext_nal_audio_head->ext_type = NAL_TYPE_EX_A_IMAADPCM;
            break;
    }
    nal_info->zero0 = 0;
    nal_info->zero1 = 0;
    nal_info->zero2 = 0;
    nal_info->constant_value_1 = 1;
    nal_info->forbidden_zero_bit = 0;
    nal_info->nal_ref_idc = NAL_PRIORITY_LOW;
    nal_info->nal_unit_type = NAL_TYPE_EX_AUDIO;
    ret = sizeof(ext_h264_nal_bitstream_t) + len;
    return ret;
}

/****************************************************************************************************************
* Function Name :   gen_ext_mjpeg_header
* Description   :   process mjpeg data encap.
* Input         :   param1 is buf
*                   param2 is buf len
* Output        :   null
* Return        :   real encode len
****************************************************************************************************************/
int  gen_ext_mjpeg_header(char *buf, int len)
{
    ext_h264_nal_bitstream_t    *ext_nal_mjpeg_head = (ext_h264_nal_bitstream_t*)buf;
    H264_NAL_INFO               *nal_info;
    int ret ;

    nal_info = &ext_nal_mjpeg_head->nal;
    nal_info->zero0 = 0;
    nal_info->zero1 = 0;
    nal_info->zero2 = 0;
    nal_info->constant_value_1 = 1;
    nal_info->forbidden_zero_bit = 0;
    nal_info->nal_ref_idc = NAL_PRIORITY_LOW;
    nal_info->nal_unit_type = NAL_TYPE_EX_PIC;
    ext_nal_mjpeg_head->ext_type = NAL_TYPE_EX_P_MJPEG;
    ret = sizeof(ext_h264_nal_bitstream_t) + len;
    return ret;
}

/****************************************************************************************************************
* Function Name :   gen_ext_mv_header
* Description   :   process mv data encap.
* Input         :   param1 is buf
*                   param2 is buf len
* Output        :   null
* Return        :   real encode len
****************************************************************************************************************/
int  gen_ext_mv_header(char *buf, int len)
{
    ext_h264_nal_bitstream_t    *ext_nal_mv_head = (ext_h264_nal_bitstream_t*)buf;
    H264_NAL_INFO               *nal_info;
    int ret ;

    nal_info = &ext_nal_mv_head->nal;
    nal_info->zero0 = 0;
    nal_info->zero1 = 0;
    nal_info->zero2 = 0;
    nal_info->constant_value_1 = 1;
    nal_info->forbidden_zero_bit = 0;
    nal_info->nal_ref_idc = NAL_PRIORITY_LOW;
    nal_info->nal_unit_type = NAL_TYPE_EX_MV;
    ext_nal_mv_head->ext_type = NAL_TYPE_EX_MV_D1;
    ret = sizeof(ext_h264_nal_bitstream_t) + len;
    return ret;
}


