/***************************************** (C) COPYRIGHT 2010 Intersil *****************************************
* File Name         : ddr_burst_interface.c
* Author            : tang.sc@intersil.com
* Revision          : 1.2.1.1275
* Date              : 2012-03-29
* Description       : 5864 all ddr transfer driver interface.
*                       5864 chip just have one dma chan for ddr burst data transfer, so all data need been queued 
*                       for serial op. 
* Change log        : 
*                   
****************************************************************************************************************
* THE PRESENT SOFTWARE IS DISTRIBUTED IN THE HOPE THAT IT WILL PROVIDE CUSTOMERS 
* WITH CODING INFORMATION TO SPEED UP THEIR PRODUCT DEVELOPMENT, BUT WITHOUT
* ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS
* FOR A PARTICULAR PURPOSE. AS A RESULT, TECHWELL SHALL NOT BE HELD LIABLE FOR ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
****************************************************************************************************************/
#include	<isil5864/isil_common.h>

/****************************************************************************************************************
* Function Name :   ddr_burst_interface_reg_init
* Description:      init ddr burst interface management struct
* Input         :   param1 is ddr burst interface management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void ddr_burst_interface_reg_init(ddr_burst_interface_t *burst_interface)
{
    if(burst_interface != NULL){
        burst_interface->page_mode_reg_value.value = 0;
        burst_interface->access_page_mode_reg_offset = MPI_ACCESS_PAGE_MODE_REG_OFFSET;
        burst_interface->cmd_status_reg_value.value = 0;
        burst_interface->burst_cmd_status_reg_offset = MPI_CMD_STATUS_REG_OFFSET;
        burst_interface->ddr_base_addr_reg_value.value = 0;
        burst_interface->burst_ddr_addr_reg_offset = MPI_BURST_DDR_ADDR_REG_OFFSET;
        burst_interface->sram_base_addr_reg_value.value = 0;
        burst_interface->burst_sram_addr_reg_offset = MPI_BURST_SRAM_ADDR_REG_OFFSET;
    }
}

/****************************************************************************************************************
* Function Name :   ddr_burst_interface_reg_reset
* Description:      reset ddr burst interface management struct
* Input         :   param1 is ddr burst interface management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void ddr_burst_interface_reg_reset(ddr_burst_interface_t *burst_interface)
{
    if(burst_interface != NULL){
        burst_interface->page_mode_reg_value.value = 0;
        burst_interface->cmd_status_reg_value.value = 0;
        burst_interface->ddr_base_addr_reg_value.value = 0;
        burst_interface->sram_base_addr_reg_value.value = 0;
    }
}

/****************************************************************************************************************
* Function Name :   ddr_burst_interface_reg_get_page_mode_reg
* Description:      get access ddr's page mode
* Input         :   param1 is ddr burst interface management struct
*                   param2 is chip management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void ddr_burst_interface_reg_get_page_mode_reg(ddr_burst_interface_t *burst_interface, isil_chip_t *chip)
{
    if((burst_interface!=NULL) && (chip!=NULL)){
        burst_interface->page_mode_reg_value.value = chip->io_op->chip_read32(chip, burst_interface->access_page_mode_reg_offset);
    }
}

/****************************************************************************************************************
* Function Name :   ddr_burst_interface_reg_update_page_mode_reg
* Description:      update ddr's page mode
* Input         :   param1 is ddr burst interface management struct
*                   param2 is chip management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void ddr_burst_interface_reg_update_page_mode_reg(ddr_burst_interface_t *burst_interface, isil_chip_t *chip)
{
    if((burst_interface!=NULL) && (chip!=NULL)){
        chip->io_op->chip_write32(chip, burst_interface->access_page_mode_reg_offset, burst_interface->page_mode_reg_value.value);
    }
}

/****************************************************************************************************************
* Function Name :   ddr_burst_interface_reg_get_burst_interface_param
* Description:      get burst interface param
* Input         :   param1 is ddr burst interface management struct
*                   param2 is chip management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void ddr_burst_interface_reg_get_burst_interface_param(ddr_burst_interface_t *burst_interface, isil_chip_t *chip)
{
    if((burst_interface!=NULL) && (chip!=NULL)){
        burst_interface->cmd_status_reg_value.value = chip->io_op->chip_read32(chip, burst_interface->burst_cmd_status_reg_offset);
        burst_interface->ddr_base_addr_reg_value.value = chip->io_op->chip_read32(chip, burst_interface->burst_ddr_addr_reg_offset);
        burst_interface->sram_base_addr_reg_value.value = chip->io_op->chip_read32(chip, burst_interface->burst_sram_addr_reg_offset);
    }
}

/****************************************************************************************************************
* Function Name :   ddr_burst_interface_reg_update_burst_interface_param
* Description:      update burst interface param
* Input         :   param1 is ddr burst interface management struct
*                   param2 is chip management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void ddr_burst_interface_reg_update_burst_interface_param(ddr_burst_interface_t *burst_interface, isil_chip_t *chip)
{
    if((burst_interface!=NULL) && (chip!=NULL)){
        if(!(chip->io_op->chip_read32(chip, MODE) & DDRON)) {
            ISIL_DBG(ISIL_DBG_ERR, "On chip DDR not enable, mode = 0x%08x\n", chip->io_op->chip_read32(chip, MODE));
            return;
        }
        if(!(chip->io_op->chip_read32(chip, MPI_ACCESS_PAGE_MODE_REG_OFFSET) & (1 << 15))) {
            ISIL_DBG(ISIL_DBG_ERR, "hardware burst interface not enable\n");
            return;
        }
        chip->io_op->chip_write32(chip, burst_interface->burst_ddr_addr_reg_offset, burst_interface->ddr_base_addr_reg_value.value);
        chip->io_op->chip_write32(chip, burst_interface->burst_sram_addr_reg_offset, burst_interface->sram_base_addr_reg_value.value);
        chip->io_op->chip_write32(chip, burst_interface->burst_cmd_status_reg_offset, burst_interface->cmd_status_reg_value.value);
        burst_interface->op->start_ddr_burst(burst_interface);
        chip->io_op->chip_write32(chip, burst_interface->burst_cmd_status_reg_offset, burst_interface->cmd_status_reg_value.value);
    }
}

/****************************************************************************************************************
* Function Name :   ddr_burst_interface_reg_enable_access_sram_block
* Description:      enable access dpram interface
* Input         :   param1 is ddr burst interface management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void ddr_burst_interface_reg_enable_access_sram_block(ddr_burst_interface_t *burst_interface)
{
    if(burst_interface != NULL){
        burst_interface->page_mode_reg_value.bit_value.burst_mode_en = 1;
    }
}

/****************************************************************************************************************
* Function Name :   ddr_burst_interface_reg_enable_access_sram_block
* Description:      enable access dpram interface
* Input         :   param1 is ddr burst interface management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void ddr_burst_interface_reg_enable_access_ddr_block(ddr_burst_interface_t *burst_interface)
{
    if(burst_interface != NULL){
        burst_interface->page_mode_reg_value.bit_value.burst_mode_en = 0;
    }
}

/****************************************************************************************************************
* Function Name :   ddr_burst_interface_reg_select_ddr_A
* Description:      select access ddr A
* Input         :   param1 is ddr burst interface management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void ddr_burst_interface_reg_select_ddr_A(ddr_burst_interface_t *burst_interface)
{
    if(burst_interface != NULL){
        burst_interface->page_mode_reg_value.bit_value.ddr_a_b_select = 0;
    }
}

/****************************************************************************************************************
* Function Name :   ddr_burst_interface_reg_select_ddr_B
* Description:      select access ddr B
* Input         :   param1 is ddr burst interface management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void ddr_burst_interface_reg_select_ddr_B(ddr_burst_interface_t *burst_interface)
{
    if(burst_interface != NULL){
        burst_interface->page_mode_reg_value.bit_value.ddr_a_b_select = 1;
    }
}

/****************************************************************************************************************
* Function Name :   ddr_burst_interface_reg_enable_read_4
* Description:      min burst len is 4DWord
* Input         :   param1 is ddr burst interface management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void ddr_burst_interface_reg_enable_read_4(ddr_burst_interface_t *burst_interface)
{
    if(burst_interface != NULL){
        burst_interface->page_mode_reg_value.bit_value.mpi_burst_read_4_enable = 1;
    }
}

/****************************************************************************************************************
* Function Name :   ddr_burst_interface_reg_disable_read_4
* Description:      min burst len is 1DWord
* Input         :   param1 is ddr burst interface management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void ddr_burst_interface_reg_disable_read_4(ddr_burst_interface_t *burst_interface)
{
    if(burst_interface != NULL){
        burst_interface->page_mode_reg_value.bit_value.mpi_burst_read_4_enable = 0;
    }
}

/****************************************************************************************************************
* Function Name :   ddr_burst_interface_reg_set_ddr_page_number
* Description:      set ddr page id
* Input         :   param1 is ddr burst interface management struct
*                   param2 is page id
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void ddr_burst_interface_reg_set_ddr_page_number(ddr_burst_interface_t *burst_interface, int page_id)
{
    if(burst_interface != NULL){
        burst_interface->page_mode_reg_value.bit_value.page_id = (page_id&0xff);
    }
}

/****************************************************************************************************************
* Function Name :   ddr_burst_interface_reg_set_ddr_burst_param
* Description:      set ddr page id
* Input         :   param1 is ddr burst interface management struct
*                   param2 is burst len
*                   param3 is burst direction
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void ddr_burst_interface_reg_set_ddr_burst_param(ddr_burst_interface_t *burst_interface, int burst_len, int dir)
{
    if(burst_interface != NULL){
        if(dir == ISIL_READ){
            burst_interface->cmd_status_reg_value.bit_value.burst_dir = 0;
        } else {
            burst_interface->cmd_status_reg_value.bit_value.burst_dir = 1;
        }
        burst_len += 3;
        burst_len >>= 2;
        burst_len <<= 2;
        burst_interface->cmd_status_reg_value.bit_value.burst_len = burst_len;
    }
}

/****************************************************************************************************************
* Function Name :   ddr_burst_interface_reg_start_ddr_burst
* Description:      enable burst transfer
* Input         :   param1 is ddr burst interface management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void ddr_burst_interface_reg_start_ddr_burst(ddr_burst_interface_t *burst_interface)
{
    if(burst_interface != NULL){
        burst_interface->cmd_status_reg_value.bit_value.burst_trigger = 1;
    }
}

/****************************************************************************************************************
* Function Name :   ddr_burst_interface_reg_int_enable
* Description:      enable burst interrupt
* Input         :   param1 is ddr burst interface management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void ddr_burst_interface_reg_int_enable(ddr_burst_interface_t *burst_interface, int ddr_single_access_err_int_enable, int ddr_burst_access_err_int_enable, int burst_end_int_enable)
{
    if(burst_interface != NULL){
        burst_interface->cmd_status_reg_value.bit_value.ddr_single_access_err_int_enable = (ddr_single_access_err_int_enable&0x1);
        burst_interface->cmd_status_reg_value.bit_value.ddr_burst_access_err_int_enable = (ddr_burst_access_err_int_enable&0x1);
        burst_interface->cmd_status_reg_value.bit_value.burst_end_int_enable = (burst_end_int_enable&0x1);
    }
}

/****************************************************************************************************************
* Function Name :   ddr_burst_interface_reg_get_single_access_err_flag
* Description:      get single access err flag
* Input         :   param1 is ddr burst interface management struct
* Output        :   null
* Return        :   single access err flag
****************************************************************************************************************/
static int  ddr_burst_interface_reg_get_single_access_err_flag(ddr_burst_interface_t *burst_interface)
{
    return burst_interface->cmd_status_reg_value.bit_value.ddr_single_access_err_flag;
}

/****************************************************************************************************************
* Function Name :   ddr_burst_interface_reg_get_burst_access_err_flag
* Description:      get burst access err flag
* Input         :   param1 is ddr burst interface management struct
* Output        :   null
* Return        :   ddr burst err flag
****************************************************************************************************************/
static int  ddr_burst_interface_reg_get_burst_access_err_flag(ddr_burst_interface_t *burst_interface)
{
    return burst_interface->cmd_status_reg_value.bit_value.ddr_burst_err_flag;
}

/****************************************************************************************************************
* Function Name :   ddr_burst_interface_reg_get_burst_busy_flag
* Description:      get burst busy flag
* Input         :   param1 is ddr burst interface management struct
* Output        :   null
* Return        :   ddr burst busy flag
****************************************************************************************************************/
static int  ddr_burst_interface_reg_get_burst_busy_flag(ddr_burst_interface_t *burst_interface)
{
    return burst_interface->cmd_status_reg_value.bit_value.ddr_burst_busy_flag;
}

/****************************************************************************************************************
* Function Name :   ddr_burst_interface_reg_get_burst_end_flag
* Description:      get burst end flag
* Input         :   param1 is ddr burst interface management struct
* Output        :   null
* Return        :   ddr burst end flag
****************************************************************************************************************/
static int  ddr_burst_interface_reg_get_burst_end_flag(ddr_burst_interface_t *burst_interface)
{
    return burst_interface->cmd_status_reg_value.bit_value.ddr_burst_end_flag;
}

/****************************************************************************************************************
* Function Name :   ddr_burst_interface_reg_clear_burst_end_flag
* Description:      clear burst end flag
* Input         :   param1 is ddr burst interface management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void ddr_burst_interface_reg_clear_burst_end_flag(ddr_burst_interface_t *burst_interface)
{
    if(burst_interface != NULL){
        burst_interface->cmd_status_reg_value.bit_value.ddr_burst_end_flag = 1;
    }
}

/****************************************************************************************************************
* Function Name :   ddr_burst_interface_reg_set_burst_ddr_addr
* Description:      set burst ddr addr
* Input         :   param1 is ddr burst interface management struct
*                   param2 is addr of ddr
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void ddr_burst_interface_reg_set_burst_ddr_addr(ddr_burst_interface_t *burst_interface, u32 addr_offset)
{
    if(burst_interface != NULL){
        burst_interface->ddr_base_addr_reg_value.bit_value.ddr_base_addr = addr_offset&0x0fffffff;
    }
}

/****************************************************************************************************************
* Function Name :   ddr_burst_interface_reg_set_burst_sram_addr
* Description:      set burst dpram addr
* Input         :   param1 is ddr burst interface management struct
*                   param2 is addr of dpram
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void ddr_burst_interface_reg_set_burst_sram_addr(ddr_burst_interface_t *burst_interface, u32 addr_offset)
{
    if(burst_interface != NULL){
        burst_interface->sram_base_addr_reg_value.bit_value.sram_base_addr = addr_offset&0x0000ffff;
    }
}

/****************************************************************************************************************
* Function Name :   ddr_burst_interface_reg_pio_host_to_sram_write
* Description:      write data into dpram based pio interface
* Input         :   param1 is ddr burst interface management struct
*                   param2 is dpram manage's node
*                   param3 is process end's addr
*                   param4 is data's len
*                   param5 is offset of dpram end
* Output        :   null
* Return        :   real transfer data len
****************************************************************************************************************/
static int  ddr_burst_interface_reg_pio_host_to_sram_write(ddr_burst_interface_t *burst_interface, dpram_page_node_t *dpram_page, u32 *host_end_addr, int len, int offset)
{
    int ret = 0;
    if((burst_interface!=NULL) && (dpram_page!=NULL) && (len!=0)){
        isil_chip_t  *chip;
        u32 dpram_page_base;
        int i;
        chip = burst_interface->chip;
        burst_interface->op->get_page_mode_reg(burst_interface, chip);
        burst_interface->op->enable_access_sram_block(burst_interface);
        burst_interface->op->update_page_mode_reg(burst_interface, chip);

        dpram_page_base = dpram_page->op->get_page_base(dpram_page);
        dpram_page_base += offset;
        ret = len;
        len += 3;
        len >>= 2;
        for(i=0; i<len; i++){
            chip->io_op->chip_write32(chip, dpram_page_base, host_end_addr[i]);
            dpram_page_base += 4;
        }
    }
    return ret;
}

/****************************************************************************************************************
* Function Name :   ddr_burst_interface_reg_dma_host_to_sram_write
* Description:      write data into dpram based dma interface
* Input         :   param1 is ddr burst interface management struct
*                   param2 is dpram manage's node
*                   param3 is process end's addr
*                   param4 is data's len
*                   param5 is offset of dpram end
* Output        :   null
* Return        :   real transfer data len
****************************************************************************************************************/
static int  ddr_burst_interface_reg_dma_host_to_sram_write(ddr_burst_interface_t *burst_interface, dpram_page_node_t *dpram_page, dma_addr_t host_end_addr, int len, int offset)
{
    int ret = 0;
    if((burst_interface!=NULL) && (dpram_page!=NULL) && (len!=0)){
        isil_chip_t  *chip;
        dma_addr_t  src, dest;
        chip = burst_interface->chip;
        burst_interface->op->get_page_mode_reg(burst_interface, chip);
        burst_interface->op->enable_access_sram_block(burst_interface);
        burst_interface->op->update_page_mode_reg(burst_interface, chip);

        dest = dpram_page->op->get_page_base(dpram_page);
        dest += chip->pa_regs;
        dest += offset;
        src = host_end_addr; 
        ret = len;
        len += 3;
        len >>= 2;
        len <<= 2;
        fpga_dma_receive((u32)src, (u32)dest, len);
    }
    return ret;
}

/****************************************************************************************************************
* Function Name :   ddr_burst_interface_reg_pio_host_to_sram_read
* Description:      read data from dpram based pio interface
* Input         :   param1 is ddr burst interface management struct
*                   param2 is dpram manage's node
*                   param3 is process end's addr
*                   param4 is data's len
*                   param5 is offset of dpram end
* Output        :   null
* Return        :   real transfer data len
****************************************************************************************************************/
static int  ddr_burst_interface_reg_pio_host_to_sram_read(ddr_burst_interface_t *burst_interface, dpram_page_node_t *dpram_page, u32 *host_end_addr, int len, int offset)
{
    int ret = 0;
    if((burst_interface!=NULL) && (dpram_page!=NULL) && (len!=0)){
        isil_chip_t  *chip;
        u32 dpram_page_base;
        int i;
        chip = burst_interface->chip;
        burst_interface->op->get_page_mode_reg(burst_interface, chip);
        burst_interface->op->enable_access_sram_block(burst_interface);
        burst_interface->op->update_page_mode_reg(burst_interface, chip);

        dpram_page_base = dpram_page->op->get_page_base(dpram_page);
        dpram_page_base += offset;
        ret = len;
        len += 3;
        len >>= 2;
        for(i=0; i<len; i++){
            host_end_addr[i] = chip->io_op->chip_read32(chip, dpram_page_base);
            dpram_page_base += 4;
        }
    }
    return ret;
}

/****************************************************************************************************************
* Function Name :   ddr_burst_interface_reg_dma_host_to_sram_read
* Description:      read data from dpram based dma interface
* Input         :   param1 is ddr burst interface management struct
*                   param2 is dpram manage's node
*                   param3 is process end's addr
*                   param4 is data's len
*                   param5 is offset of dpram end
* Output        :   null
* Return        :   real transfer data len
****************************************************************************************************************/
static int  ddr_burst_interface_reg_dma_host_to_sram_read(ddr_burst_interface_t *burst_interface, dpram_page_node_t *dpram_page, dma_addr_t host_end_addr, int len, int offset)
{
    int ret=0;
    if((burst_interface!=NULL) && (dpram_page!=NULL) && (len!=0)){
        isil_chip_t  *chip;
        dma_addr_t  src, dest;
        chip = burst_interface->chip;
        burst_interface->op->get_page_mode_reg(burst_interface, chip);
        burst_interface->op->enable_access_sram_block(burst_interface);
        burst_interface->op->update_page_mode_reg(burst_interface, chip);

        src = dpram_page->op->get_page_base(dpram_page);
        src += chip->pa_regs;
        src += offset;
        dest = host_end_addr; 
        ret = len;
        len += 3;
        len >>= 2;
        len <<= 2;
        fpga_dma_receive((u32)src, (u32)dest, len);
    }
    return ret;
}

/****************************************************************************************************************
* Function Name :   ddr_burst_interface_reg_start_block_transfer_from_sram_to_ddr
* Description:      block transfer data from dpram to ddr
* Input         :   param1 is ddr burst interface management struct
*                   param2 is dpram manage's node
*                   param3 is ddr end's addr
*                   param4 is ddr page id
*                   param5 is data len
*                   param6 is ddr A or B
* Output        :   null
* Return        :   real transfer data len
****************************************************************************************************************/
static int  ddr_burst_interface_reg_start_block_transfer_from_sram_to_ddr(ddr_burst_interface_t *burst_interface, dpram_page_node_t *dpram_page, u32 ddr_end_addr, int page_id, int len, int ddr_chip_a_or_b)
{
    int count, escape;

    count = escape = 0;
    if((burst_interface!=NULL) && (dpram_page!=NULL) && (len!=0)){
        isil_chip_t  *chip;
        u32 dpram_page_base;
        chip = burst_interface->chip;
        burst_interface->op->get_page_mode_reg(burst_interface, chip);
        burst_interface->op->enable_access_sram_block(burst_interface);
        burst_interface->op->update_page_mode_reg(burst_interface, chip);

        count = escape = 0;
        burst_interface->op->get_burst_interface_param(burst_interface, chip);
        while(burst_interface->op->get_burst_busy_flag(burst_interface)){
            burst_interface->op->get_burst_interface_param(burst_interface, chip);
            count++;
            if(count > BURST_INTERFACE_CHECKUP_TIMEOUT){
                ISIL_DBG(ISIL_DBG_INFO, ": can't access burst interface\n");
                escape = 1;
                break;
            }
        }

        if(!escape){
            burst_interface->op->set_ddr_page_number(burst_interface, page_id);
            if(DDR_CHIP_B == ddr_chip_a_or_b){
                burst_interface->op->select_ddr_B(burst_interface);
            } else {
                burst_interface->op->select_ddr_A(burst_interface); 
            }
#if defined(PCI_PLATFORM) && defined(DMA_INTERFACE) && defined(BURST_INTERFACE)
            burst_interface->op->enable_read_4(burst_interface);
#else
            burst_interface->op->disable_read_4(burst_interface);
#endif
            burst_interface->op->update_page_mode_reg(burst_interface, chip);

            dpram_page_base = dpram_page->op->get_page_offset(dpram_page);
            burst_interface->op->set_burst_sram_addr(burst_interface, dpram_page_base);
            burst_interface->op->set_burst_ddr_addr(burst_interface, ddr_end_addr);
            burst_interface->op->int_enable(burst_interface, 0, 0, 0);
            burst_interface->op->set_ddr_burst_param(burst_interface, len, ISIL_WRITE);
            burst_interface->op->update_burst_interface_param(burst_interface, chip);

            count = escape = 0;
            burst_interface->op->get_burst_interface_param(burst_interface, chip);
            while(burst_interface->op->get_burst_end_flag(burst_interface) == 0){
                burst_interface->op->get_burst_interface_param(burst_interface, chip);
                count++;
                if(count > BURST_INTERFACE_CHECKUP_TIMEOUT){
                    ISIL_DBG(ISIL_DBG_INFO, ": can't access burst interface\n");
                    escape = 1;
                    break;
                }
            }

            if(escape == 0){
                burst_interface->op->clear_burst_end_flag(burst_interface);
                burst_interface->op->update_burst_interface_param(burst_interface, chip);
            }
        }
        burst_interface->op->clear_burst_done(burst_interface);
    }
    return escape;
}

/****************************************************************************************************************
* Function Name :   ddr_burst_interface_reg_start_nonblock_transfer_from_sram_to_ddr
* Description:      nonblock transfer data from dpram to ddr
* Input         :   param1 is ddr burst interface management struct
*                   param2 is dpram manage's node
*                   param3 is ddr end's addr
*                   param4 is ddr page id
*                   param5 is data len
*                   param6 is ddr A or B
* Output        :   null
* Return        :   real transfer data len
****************************************************************************************************************/
static int  ddr_burst_interface_reg_start_nonblock_transfer_from_sram_to_ddr(ddr_burst_interface_t *burst_interface, dpram_page_node_t *dpram_page, u32 ddr_end_addr, int page_id, int len, int ddr_chip_a_or_b)
{
    int count, escape;

    count = escape = 0;
    if((burst_interface!=NULL) && (dpram_page!=NULL) && (len!=0)){
        isil_chip_t  *chip;
        u32 dpram_page_base;
        int count, escape;

        chip = burst_interface->chip;
        burst_interface->op->get_page_mode_reg(burst_interface, chip);
        burst_interface->op->enable_access_sram_block(burst_interface);
        burst_interface->op->update_page_mode_reg(burst_interface, chip);

        count = escape = 0;
        burst_interface->op->get_burst_interface_param(burst_interface, chip);
        while(burst_interface->op->get_burst_busy_flag(burst_interface)){
            burst_interface->op->get_burst_interface_param(burst_interface, chip);
            count++;
            if(count > BURST_INTERFACE_CHECKUP_TIMEOUT){
                ISIL_DBG(ISIL_DBG_INFO, ": can't access burst interface\n");
                escape = 1;
                break;
            }
        }

        if(!escape){
            burst_interface->op->set_ddr_page_number(burst_interface, page_id);
            if(DDR_CHIP_B == ddr_chip_a_or_b){
                burst_interface->op->select_ddr_B(burst_interface);
            } else {
                burst_interface->op->select_ddr_A(burst_interface); 
            }
#if defined(PCI_PLATFORM) && defined(DMA_INTERFACE) && defined(BURST_INTERFACE)
            burst_interface->op->enable_read_4(burst_interface);
#else
            burst_interface->op->disable_read_4(burst_interface);
#endif
            burst_interface->op->update_page_mode_reg(burst_interface, chip);

            dpram_page_base = dpram_page->op->get_page_offset(dpram_page);
            burst_interface->op->set_burst_sram_addr(burst_interface, dpram_page_base);
            burst_interface->op->set_burst_ddr_addr(burst_interface, ddr_end_addr);
            burst_interface->op->int_enable(burst_interface, 0, 0, 1);
            burst_interface->op->set_ddr_burst_param(burst_interface, len, ISIL_WRITE);
            burst_interface->op->update_burst_interface_param(burst_interface, chip);
        }
    }
    return escape;
}

/****************************************************************************************************************
* Function Name :   ddr_burst_interface_reg_start_block_transfer_from_ddr_to_sram
* Description:      block transfer data from ddr to dpram
* Input         :   param1 is ddr burst interface management struct
*                   param2 is dpram manage's node
*                   param3 is ddr end's addr
*                   param4 is ddr page id
*                   param5 is data len
*                   param6 is ddr A or B
* Output        :   null
* Return        :   real transfer data len
****************************************************************************************************************/
static int  ddr_burst_interface_reg_start_block_transfer_from_ddr_to_sram(ddr_burst_interface_t *burst_interface, dpram_page_node_t *dpram_page, u32 ddr_end_addr, int page_id, int len, int ddr_chip_a_or_b)
{
    int count, escape;

    count = escape = 0;
    if((burst_interface!=NULL) && (dpram_page!=NULL) && (len!=0)){
        isil_chip_t  *chip;
        u32 dpram_page_base;

        chip = burst_interface->chip;
        burst_interface->op->get_page_mode_reg(burst_interface, chip);
        burst_interface->op->enable_access_sram_block(burst_interface);
        burst_interface->op->update_page_mode_reg(burst_interface, chip);
        count = escape = 0;
        burst_interface->op->get_burst_interface_param(burst_interface, chip);
        while(burst_interface->op->get_burst_busy_flag(burst_interface)){
            burst_interface->op->get_burst_interface_param(burst_interface, chip);
            count++;
            if(count > BURST_INTERFACE_CHECKUP_TIMEOUT){
                ISIL_DBG(ISIL_DBG_INFO, ": can't access burst interface\n");
                escape = 1;
                break;
            }
        }

        if(!escape){
            burst_interface->op->set_ddr_page_number(burst_interface, page_id);
            if(DDR_CHIP_B == ddr_chip_a_or_b){
                burst_interface->op->select_ddr_B(burst_interface);
            } else {
                burst_interface->op->select_ddr_A(burst_interface); 
            }
#if defined(PCI_PLATFORM) && defined(DMA_INTERFACE) && defined(BURST_INTERFACE)
            burst_interface->op->enable_read_4(burst_interface);
#else
            burst_interface->op->disable_read_4(burst_interface);
#endif
            burst_interface->op->update_page_mode_reg(burst_interface, chip);

            dpram_page_base = dpram_page->op->get_page_offset(dpram_page);
            burst_interface->op->set_burst_sram_addr(burst_interface, dpram_page_base);
            burst_interface->op->set_burst_ddr_addr(burst_interface, ddr_end_addr);
            burst_interface->op->int_enable(burst_interface, 0, 0, 0);
            burst_interface->op->set_ddr_burst_param(burst_interface, len, ISIL_READ);
            burst_interface->op->update_burst_interface_param(burst_interface, chip);
            count = escape = 0;
            burst_interface->op->get_burst_interface_param(burst_interface, chip);
            while(burst_interface->op->get_burst_end_flag(burst_interface) == 0){
                burst_interface->op->get_burst_interface_param(burst_interface, chip);
                count++;
                if(count > BURST_INTERFACE_CHECKUP_TIMEOUT){
                    ISIL_DBG(ISIL_DBG_INFO, ": can't access burst interface\n");
                    escape = 1;
                    break;
                }
            }

            if(escape == 0){
                burst_interface->op->clear_burst_end_flag(burst_interface);
                burst_interface->op->update_burst_interface_param(burst_interface, chip);
            }
        }
        burst_interface->op->clear_burst_done(burst_interface);
    }
    return escape;
}

/****************************************************************************************************************
* Function Name :   ddr_burst_interface_reg_start_nonblock_transfer_from_ddr_to_sram
* Description:      nonblock transfer data from ddr to dpram
* Input         :   param1 is ddr burst interface management struct
*                   param2 is dpram manage's node
*                   param3 is ddr end's addr
*                   param4 is ddr page id
*                   param5 is data len
*                   param6 is ddr A or B
* Output        :   null
* Return        :   real transfer data len
****************************************************************************************************************/
static int  ddr_burst_interface_reg_start_nonblock_transfer_from_ddr_to_sram(ddr_burst_interface_t *burst_interface, dpram_page_node_t *dpram_page, u32 ddr_end_addr, int page_id, int len, int ddr_chip_a_or_b)
{
    int count, escape;

    count = escape = 0;
    if((burst_interface!=NULL) && (dpram_page!=NULL) && (len!=0)){
        isil_chip_t  *chip;
        u32 dpram_page_base;
        chip = burst_interface->chip;
        burst_interface->op->get_page_mode_reg(burst_interface, chip);
        burst_interface->op->enable_access_sram_block(burst_interface);
        burst_interface->op->update_page_mode_reg(burst_interface, chip);

        count = escape = 0;
        burst_interface->op->get_burst_interface_param(burst_interface, chip);
        while(burst_interface->cmd_status_reg_value.bit_value.ddr_burst_busy_flag || burst_interface->cmd_status_reg_value.bit_value.ddr_single_access_busy_flag){
            burst_interface->op->get_burst_interface_param(burst_interface, chip);
            count++;
            if(count > BURST_INTERFACE_CHECKUP_TIMEOUT){
                ISIL_DBG(ISIL_DBG_INFO, ": can't access burst interface\n");
                escape = 1;
                break;
            }
        }

        if(!escape){
            burst_interface->op->set_ddr_page_number(burst_interface, page_id);
            if(DDR_CHIP_B == ddr_chip_a_or_b){
                burst_interface->op->select_ddr_B(burst_interface);
            } else {
                burst_interface->op->select_ddr_A(burst_interface); 
            }
#if defined(PCI_PLATFORM) && defined(DMA_INTERFACE) && defined(BURST_INTERFACE)
            burst_interface->op->enable_read_4(burst_interface);
#else
            burst_interface->op->disable_read_4(burst_interface);
#endif
            burst_interface->op->update_page_mode_reg(burst_interface, chip);

            dpram_page_base = dpram_page->op->get_page_offset(dpram_page);
            burst_interface->op->set_burst_sram_addr(burst_interface, dpram_page_base);
            burst_interface->op->set_burst_ddr_addr(burst_interface, ddr_end_addr);
            burst_interface->op->int_enable(burst_interface, 0, 0, 1);
            burst_interface->op->set_ddr_burst_param(burst_interface, len, ISIL_READ);
            burst_interface->op->update_burst_interface_param(burst_interface, chip);
        }
    }
    return escape;
}

/****************************************************************************************************************
* Function Name :   ddr_burst_interface_reg_clear_burst_done
* Description:      clear burst done flag
* Input         :   param1 is ddr burst interface management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void ddr_burst_interface_reg_clear_burst_done(ddr_burst_interface_t *burst_interface)
{
    if(burst_interface != NULL){
        isil_chip_t      *chip;
        chip = burst_interface->chip;
        burst_interface->op->get_page_mode_reg(burst_interface, chip);
        burst_interface->op->enable_access_sram_block(burst_interface);
        burst_interface->op->update_page_mode_reg(burst_interface, chip);

        burst_interface->op->get_burst_interface_param(burst_interface, chip);
        burst_interface->op->clear_burst_end_flag(burst_interface);
        chip->io_op->chip_write32(chip, burst_interface->burst_cmd_status_reg_offset, burst_interface->cmd_status_reg_value.value);
        burst_interface->op->get_burst_interface_param(burst_interface, chip);
    }
}

/****************************************************************************************************************
* Veriable Name :   ddr_burst_interface_reg_op
* Description   :   for op burst interface, we define general interface
****************************************************************************************************************/
static struct ddr_burst_interface_reg_operation ddr_burst_interface_reg_op = {
    .init = ddr_burst_interface_reg_init,
    .reset = ddr_burst_interface_reg_reset,
    .get_page_mode_reg = ddr_burst_interface_reg_get_page_mode_reg,
    .update_page_mode_reg = ddr_burst_interface_reg_update_page_mode_reg,
    .get_burst_interface_param = ddr_burst_interface_reg_get_burst_interface_param,
    .update_burst_interface_param = ddr_burst_interface_reg_update_burst_interface_param,

    .enable_access_sram_block = ddr_burst_interface_reg_enable_access_sram_block,
    .enable_access_ddr_block = ddr_burst_interface_reg_enable_access_ddr_block,
    .select_ddr_A = ddr_burst_interface_reg_select_ddr_A,
    .select_ddr_B = ddr_burst_interface_reg_select_ddr_B,
    .enable_read_4 = ddr_burst_interface_reg_enable_read_4,
    .disable_read_4 = ddr_burst_interface_reg_disable_read_4,
    .set_ddr_page_number = ddr_burst_interface_reg_set_ddr_page_number,

    .set_ddr_burst_param = ddr_burst_interface_reg_set_ddr_burst_param,
    .start_ddr_burst = ddr_burst_interface_reg_start_ddr_burst,
    .int_enable = ddr_burst_interface_reg_int_enable,
    .get_single_access_err_flag = ddr_burst_interface_reg_get_single_access_err_flag,
    .get_burst_access_err_flag = ddr_burst_interface_reg_get_burst_access_err_flag,
    .get_burst_busy_flag = ddr_burst_interface_reg_get_burst_busy_flag,
    .get_burst_end_flag = ddr_burst_interface_reg_get_burst_end_flag,
    .clear_burst_end_flag = ddr_burst_interface_reg_clear_burst_end_flag,
    .set_burst_ddr_addr = ddr_burst_interface_reg_set_burst_ddr_addr,
    .set_burst_sram_addr = ddr_burst_interface_reg_set_burst_sram_addr,

    .pio_host_to_sram_write = ddr_burst_interface_reg_pio_host_to_sram_write,
    .dma_host_to_sram_write = ddr_burst_interface_reg_dma_host_to_sram_write,
    .pio_host_to_sram_read = ddr_burst_interface_reg_pio_host_to_sram_read,
    .dma_host_to_sram_read = ddr_burst_interface_reg_dma_host_to_sram_read,

    .start_block_transfer_from_sram_to_ddr = ddr_burst_interface_reg_start_block_transfer_from_sram_to_ddr,
    .start_nonblock_transfer_from_sram_to_ddr = ddr_burst_interface_reg_start_nonblock_transfer_from_sram_to_ddr,
    .start_block_transfer_from_ddr_to_sram = ddr_burst_interface_reg_start_block_transfer_from_ddr_to_sram,
    .start_nonblock_transfer_from_ddr_to_sram = ddr_burst_interface_reg_start_nonblock_transfer_from_ddr_to_sram,
    .clear_burst_done = ddr_burst_interface_reg_clear_burst_done,
};

/****************************************************************************************************************
* Function Name :   init_ddr_burst_interface_t
* Description:      init ddr burst interface
* Input         :   param1 is ddr burst interface management struct
*                   param2 is chip management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void    init_ddr_burst_interface_t(ddr_burst_interface_t *burst_interface, isil_chip_t *chip)
{
    if(burst_interface != NULL){
        burst_interface->op = &ddr_burst_interface_reg_op;
        burst_interface->chip = chip;
        burst_interface->op->init(burst_interface);
        burst_interface->op->update_page_mode_reg(burst_interface, chip);
        burst_interface->op->update_burst_interface_param(burst_interface, chip);
    }
}

/****************************************************************************************************************
* Function Name :   dpram_pgae_node_get_page_base
* Description:      get dpram page node's base address
* Input         :   param1 is dpram page management struct
* Output        :   null
* Return        :   base address
****************************************************************************************************************/
static unsigned long  dpram_pgae_node_get_page_base(dpram_page_node_t *dpram_page)
{
    return (dpram_page->dpram_base + dpram_page->dpram_page_base_offset);
}

/****************************************************************************************************************
* Function Name :   dpram_page_node_get_page_offset
* Description:      get dpram page node's address offset
* Input         :   param1 is dpram page management struct
* Output        :   null
* Return        :   address offset
****************************************************************************************************************/
static unsigned long  dpram_page_node_get_page_offset(dpram_page_node_t *dpram_page)
{
    return dpram_page->dpram_page_base_offset;
}

/****************************************************************************************************************
* Function Name :   dpram_page_node_get_page_size
* Description:      get dpram page node's size
* Input         :   param1 is dpram page management struct
* Output        :   null
* Return        :   size
****************************************************************************************************************/
static int  dpram_page_node_get_page_size(dpram_page_node_t *dpram_page)
{
    return dpram_page->page_size;
}

/****************************************************************************************************************
* Veriable Name :   dpram_page_node_op
* Description   :   for op dpram page, we define general interface
****************************************************************************************************************/
static struct dpram_page_node_operation dpram_page_node_op = {
    .get_page_base = dpram_pgae_node_get_page_base,
    .get_page_offset = dpram_page_node_get_page_offset,
    .get_page_size = dpram_page_node_get_page_size,
};

/****************************************************************************************************************
* Function Name :   dpram_page_resource_manage_init
* Description:      dpram page resource management init
* Input         :   param1 is dpram page resource management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void dpram_page_resource_manage_init(dpram_page_resource_manage_t *dpram_page_resource)
{
    if(dpram_page_resource){
        dpram_page_node_t   *dpram;

        dpram = dpram_page_resource->dpram;
        dpram->dpram_base = MPI_DPRAM_BASE_OFFSET;
        dpram->dpram_page_base_offset = 0;
        dpram->page_size = DPRAM_SEGMENT_SIZE;
        dpram->op = &dpram_page_node_op;
    }
}

/****************************************************************************************************************
* Function Name :   dpram_page_resource_manage_get_dpram_page
* Description:      get a dpram page node
* Input         :   param1 is dpram page resource management struct
* Output        :   dpram page node
* Return        :   null
****************************************************************************************************************/
static void dpram_page_resource_manage_get_dpram_page(dpram_page_resource_manage_t *dpram_page_resource, dpram_page_node_t **ptr_dpram_page)
{
    if((dpram_page_resource!=NULL) && (ptr_dpram_page!=NULL)){
        *ptr_dpram_page = dpram_page_resource->dpram;
    }
}

/****************************************************************************************************************
* Veriable Name :   dpram_page_resource_manage_op
* Description   :   for op dpram page resource manage, we define general interface
****************************************************************************************************************/
static struct dpram_page_resource_manage_operation  dpram_page_resource_manage_op = {
    .init = dpram_page_resource_manage_init,
    .get_dpram_page = dpram_page_resource_manage_get_dpram_page,
};

/****************************************************************************************************************
* Function Name :   init_dpram_page_resource_manage
* Description:      init dpram page resource
* Input         :   param1 is dpram page resource management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void    init_dpram_page_resource_manage(dpram_page_resource_manage_t *dpram_page_resource)
{
    if(dpram_page_resource != NULL){
        dpram_page_resource->op = &dpram_page_resource_manage_op;
        dpram_page_resource->op->init(dpram_page_resource);
    }
}

/****************************************************************************************************************
* Function Name :   init_dpram_request_service_tcb
* Description:      init dpram request service task control block
* Input         :   param1 is dpram request tcb
* Output        :   null
* Return        :   null
****************************************************************************************************************/
void    init_dpram_request_service_tcb(dpram_request_t *request_tcb)
{
    if(request_tcb != NULL){
        tcb_node_t  *node = &request_tcb->request_node;
        request_tcb->chip = NULL;
        request_tcb->type = DPRAM_BLOCK_TRANSFER_REQUEST;
        request_tcb->context = NULL;
        request_tcb->req_callback = NULL;
        node->op = &tcb_node_op;
        node->op->init(node);
        node->op->set_priv(node, request_tcb);
    }
}

/****************************************************************************************************************
* Function Name :   dpram_request_queue_init
* Description:      init dpram request service queue
* Input         :   param1 is dpram request queue
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void dpram_request_queue_init(dpram_request_queue_t *request_queue)
{
    if(request_queue != NULL){
        tcb_node_queue_t    *request_queue_node = &request_queue->request_queue;
        request_queue->curr_service_req = NULL;
        spin_lock_init(&request_queue->dpram_request_queue_lock);
        request_queue_node->op = &tcb_node_queue_op;
        request_queue_node->op->init(request_queue_node);
    }
}

/****************************************************************************************************************
* Function Name :   dpram_request_queue_get_queue_curr_entry_number
* Description:      get curr number of queue
* Input         :   param1 is dpram request queue
* Output        :   null
* Return        :   real number
****************************************************************************************************************/
static int  dpram_request_queue_get_queue_curr_entry_number(dpram_request_queue_t *request_queue)
{
    if(request_queue != NULL) {
        tcb_node_queue_t    *request_queue_node = &request_queue->request_queue;
        return request_queue_node->op->get_queue_curr_entry_number(request_queue_node);
    } else {
        return 0;
    }
}

/****************************************************************************************************************
* Function Name :   dpram_request_queue_put_service_request_into_queue
* Description:      put a service request tcb into queue
* Input         :   param1 is dpram request queue
*                   param2 is dpram request
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void dpram_request_queue_put_service_request_into_queue(dpram_request_queue_t *request_queue, dpram_request_t *request)
{
    if((request_queue!=NULL) && (request!=NULL)){
        tcb_node_queue_t    *request_queue_node = &request_queue->request_queue;
        request_queue_node->op->put(request_queue_node, &request->request_node);
        request_queue->op->trigger_chip_pending_service_request(request_queue);
    }
}

/****************************************************************************************************************
* Function Name :   dpram_request_queue_put_service_request_into_queue_header
* Description:      put a service request tcb into queue
* Input         :   param1 is dpram request queue
*                   param2 is dpram request
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void dpram_request_queue_put_service_request_into_queue_header(dpram_request_queue_t *request_queue, dpram_request_t *request)
{
    if((request_queue!=NULL) && (request!=NULL)){
        tcb_node_queue_t    *request_queue_node = &request_queue->request_queue;
        request_queue_node->op->put_header(request_queue_node, &request->request_node);
        request_queue->op->trigger_chip_pending_service_request(request_queue);
    }
}

/****************************************************************************************************************
* Function Name :   dpram_request_queue_delete_service_request_from_queue
* Description:      delete a service request tcb from queue
* Input         :   param1 is dpram request queue
*                   param2 is dpram request
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static int  dpram_request_queue_delete_service_request_from_queue(dpram_request_queue_t *request_queue, dpram_request_t *request)
{
    int ret = 0;

    if((request_queue!=NULL) && (request!=NULL)){
        tcb_node_queue_t    *request_queue_node = &request_queue->request_queue;
        unsigned long   flags;
        spin_lock_irqsave(&request_queue->dpram_request_queue_lock, flags);
        if(request_queue->op->get_queue_curr_entry_number(request_queue)){
            ret = request_queue_node->op->delete(request_queue_node, &request->request_node);
            if(ret == 1){
                if(request->delete_req_notify != NULL){
                    request->delete_req_notify(request, request->context);
                }
            }
        }
        spin_unlock_irqrestore(&request_queue->dpram_request_queue_lock, flags);
    }
    return ret;
}

/****************************************************************************************************************
* Function Name :   dpram_request_queue_try_get_curr_consumer_from_queue
* Description:      try get a service request tcb from queue
* Input         :   param1 is dpram request queue
*                   param2 is dpram request
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void dpram_request_queue_try_get_curr_consumer_from_queue(dpram_request_queue_t *request_queue)
{
    if(request_queue != NULL){
        tcb_node_queue_t    *request_queue_node = &request_queue->request_queue;
        if(request_queue->curr_service_req == NULL){
            if(request_queue_node->op != NULL){
                tcb_node_t  *temp_node;
                request_queue_node->op->try_get(request_queue_node, &temp_node);
                if(temp_node != NULL){
                    request_queue->curr_service_req = to_get_dpram_request_with_request_node(temp_node);
                }
            }
        }
    }
}

/****************************************************************************************************************
* Function Name :   dpram_request_queue_release_curr_consumer
* Description:      release curr consumer of queue
* Input         :   param1 is dpram request queue
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void dpram_request_queue_release_curr_consumer(dpram_request_queue_t *request_queue)
{
    if(request_queue != NULL){
        unsigned long	flags;
        spin_lock_irqsave(&request_queue->dpram_request_queue_lock, flags);
        request_queue->curr_service_req = NULL;
        spin_unlock_irqrestore(&request_queue->dpram_request_queue_lock, flags);
    }
}

/****************************************************************************************************************
* Function Name :   dpram_request_queue_trigger_chip_pending_service_request
* Description:      trigger requst dpram tcb
* Input         :   param1 is dpram request queue
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void dpram_request_queue_trigger_chip_pending_service_request(dpram_request_queue_t *request_queue)
{
    if(request_queue != NULL){
        isil_chip_t     *chip;
        dpram_request_t *curr_request;
        unsigned long   flags;
        int curr_entry_number;

again:
        spin_lock_irqsave(&request_queue->dpram_request_queue_lock, flags);
        curr_entry_number = request_queue->op->get_queue_curr_entry_number(request_queue);
        if((curr_entry_number==0) && (request_queue->curr_service_req==NULL)){
            spin_unlock_irqrestore(&request_queue->dpram_request_queue_lock, flags);
            return;
        } else if(request_queue->curr_service_req != NULL){
            spin_unlock_irqrestore(&request_queue->dpram_request_queue_lock, flags);
            return;
        }
        request_queue->op->try_get_curr_consumer_from_queue(request_queue);
        curr_request = request_queue->curr_service_req;
        spin_unlock_irqrestore(&request_queue->dpram_request_queue_lock, flags);

        if(curr_request != NULL){
            chip = curr_request->chip;
            if(chip != NULL){
                if(curr_request->req_callback != NULL){
                    curr_request->req_callback(curr_request, curr_request->context);
                } else {
                    if(curr_request->delete_req_notify != NULL){
                        curr_request->delete_req_notify(curr_request, curr_request->context);
                    }
                    request_queue->op->release_curr_consumer(request_queue);
                    ISIL_DBG(ISIL_DBG_INFO, ": &&&&&&&&&&&&,%d\n\n", curr_entry_number);
                }
            } else {
                request_queue->op->release_curr_consumer(request_queue);
                ISIL_DBG(ISIL_DBG_INFO, "\n");
                goto again;
            }
        }
    }
}

/****************************************************************************************************************
* Function Name :   dpram_request_queue_service_request_queue_is_empty
* Description:      whether or not the queue have any entry
* Input         :   param1 is dpram request queue
* Output        :   null
* Return        :   real counter
****************************************************************************************************************/
static int  dpram_request_queue_service_request_queue_is_empty(dpram_request_queue_t *request_queue)
{
    if(request_queue != NULL) {
        unsigned long   flags;
        int curr_entry_number;

        spin_lock_irqsave(&request_queue->dpram_request_queue_lock, flags);
        curr_entry_number = request_queue->op->get_queue_curr_entry_number(request_queue);
        if((curr_entry_number==0) && (request_queue->curr_service_req==NULL)){
            curr_entry_number = 1;
        } else {
            curr_entry_number = 0;
        }
        spin_unlock_irqrestore(&request_queue->dpram_request_queue_lock, flags);
        return curr_entry_number;
    } else {
        return 0;
    }
}

/****************************************************************************************************************
* Veriable Name :   dpram_request_queue_op
* Description   :   for op dpram request queue, we define general interface
****************************************************************************************************************/
static struct dpram_request_queue_operation dpram_request_queue_op = {
    .init = dpram_request_queue_init,
    .get_queue_curr_entry_number = dpram_request_queue_get_queue_curr_entry_number,
    .put_service_request_into_queue = dpram_request_queue_put_service_request_into_queue,
    .put_service_request_into_queue_header = dpram_request_queue_put_service_request_into_queue_header,
    .delete_service_request_from_queue = dpram_request_queue_delete_service_request_from_queue,
    .try_get_curr_consumer_from_queue = dpram_request_queue_try_get_curr_consumer_from_queue,
    .release_curr_consumer = dpram_request_queue_release_curr_consumer,
    .trigger_chip_pending_service_request = dpram_request_queue_trigger_chip_pending_service_request,
    .service_request_queue_is_empty = dpram_request_queue_service_request_queue_is_empty,
};

/****************************************************************************************************************
* Function Name :   init_dpram_request_queue
* Description:      init dpram request queue
* Input         :   param1 is dpram request queue
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void    init_dpram_request_queue(dpram_request_queue_t *request_queue)
{
    if(request_queue != NULL){
        request_queue->op = &dpram_request_queue_op;
        request_queue->op->init(request_queue);
    }
}

/****************************************************************************************************************
* Function Name :   chip_ddr_burst_engine_init
* Description:      chip ddr burst engine init
* Input         :   param1 is chip ddr burst engine management struct
*                   param2 is chip management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void chip_ddr_burst_engine_init(chip_ddr_burst_engine_t *chip_ddr_burst_interface, isil_chip_t *chip)
{
    if((chip_ddr_burst_interface!=NULL) && (chip!=NULL)){
        atomic_set(&chip_ddr_burst_interface->bus_state, BUS_IDLE);
		init_dpram_request_queue(&chip_ddr_burst_interface->dpram_request_queue);
        init_ddr_burst_interface_t(&chip_ddr_burst_interface->burst_interface, chip);
        init_dpram_page_resource_manage(&chip_ddr_burst_interface->dpram_resource);
    }
}

/****************************************************************************************************************
* Function Name :   chip_ddr_burst_engine_get_dpram_request_queue
* Description:      get dpram request queue
* Input         :   param1 is chip ddr burst engine management struct
* Output        :   dpram request queue
* Return        :   null
****************************************************************************************************************/
static void	chip_ddr_burst_engine_get_dpram_request_queue(chip_ddr_burst_engine_t *chip_ddr_burst_interface, dpram_request_queue_t **ptr_dpram_request_queue)
{
    if(chip_ddr_burst_interface != NULL){
        *ptr_dpram_request_queue = &chip_ddr_burst_interface->dpram_request_queue;
    }
}

/****************************************************************************************************************
* Function Name :   chip_ddr_burst_engine_get_ddr_burst_interface
* Description:      get a burst interface
* Input         :   param1 is chip ddr burst engine management struct
* Output        :   burst interface management struct
* Return        :   null
****************************************************************************************************************/
static void chip_ddr_burst_engine_get_ddr_burst_interface(chip_ddr_burst_engine_t *chip_ddr_burst_interface, ddr_burst_interface_t **ptr_burst_interface)
{
    if(chip_ddr_burst_interface != NULL){
        *ptr_burst_interface = &chip_ddr_burst_interface->burst_interface;
    }
}

/****************************************************************************************************************
* Function Name :   chip_ddr_burst_engine_get_dpram_page_resource_manage
* Description:      get dpram resource management struct
* Input         :   param1 is chip ddr burst engine management struct
* Output        :   dpram resource management struct
* Return        :   null
****************************************************************************************************************/
static void chip_ddr_burst_engine_get_dpram_page_resource_manage(chip_ddr_burst_engine_t *chip_ddr_burst_interface, dpram_page_resource_manage_t **ptr_dpram_resource)
{
    if(chip_ddr_burst_interface != NULL){
        *ptr_dpram_resource = &chip_ddr_burst_interface->dpram_resource;
    }
}

/****************************************************************************************************************
* Function Name :   chip_ddr_burst_engine_is_can_submit_move_data_from_host_to_dpram_service_req
* Description:      whether or not submit data transfer from host to dpram
* Input         :   param1 is chip ddr burst engine management struct
*                   param2 is dpram page node management strcut
* Output        :   null
* Return        :   1 is meaning ok, other meaning can't
****************************************************************************************************************/
static int  chip_ddr_burst_engine_is_can_submit_move_data_from_host_to_dpram_service_req(chip_ddr_burst_engine_t *chip_ddr_burst_interface, dpram_page_node_t **ptr_dpram_page)
{
    int ret = 0;
    if(chip_ddr_burst_interface != NULL){
        dpram_page_resource_manage_t *dpram_resource;
        int bus_state;

        bus_state = atomic_read(&chip_ddr_burst_interface->bus_state);
        if(!(bus_state&DDR_END_BUS_BUSY)){
			atomic_set(&chip_ddr_burst_interface->bus_state, (bus_state|DDR_END_BUS_BUSY));
            dpram_resource = &chip_ddr_burst_interface->dpram_resource;
            dpram_resource->op->get_dpram_page(dpram_resource, ptr_dpram_page);
			ret = 1;
        }
    }
    return ret;
}

/****************************************************************************************************************
* Function Name :   chip_ddr_burst_engine_notify_end_move_data_from_dpram_to_ddr_service_req
* Description:      notify the transfer from dpram to ddr is over
* Input         :   param1 is chip ddr burst engine management struct
*                   param2 is dpram page node management strcut
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void chip_ddr_burst_engine_notify_end_move_data_from_dpram_to_ddr_service_req(chip_ddr_burst_engine_t *chip_ddr_burst_interface, dpram_page_node_t **ptr_dpram_page)
{
    if(chip_ddr_burst_interface != NULL){
        dpram_request_queue_t       *dpram_request_queue;

		if(chip_ddr_burst_interface->op != NULL){
			chip_ddr_burst_interface->op->release_end_move_data_from_dpram_to_ddr_service_req(chip_ddr_burst_interface, ptr_dpram_page);
		}
		dpram_request_queue = &chip_ddr_burst_interface->dpram_request_queue;
		dpram_request_queue->op->release_curr_consumer(dpram_request_queue);
		dpram_request_queue->op->trigger_chip_pending_service_request(dpram_request_queue);
    }
}

/****************************************************************************************************************
* Function Name :   chip_ddr_burst_engine_release_end_move_data_from_dpram_to_ddr_service_req
* Description:      release the transfer from dpram to ddr is over
* Input         :   param1 is chip ddr burst engine management struct
*                   param2 is dpram page node management strcut
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void chip_ddr_burst_engine_release_end_move_data_from_dpram_to_ddr_service_req(chip_ddr_burst_engine_t *chip_ddr_burst_interface, dpram_page_node_t **ptr_dpram_page)
{
    if(chip_ddr_burst_interface != NULL){
        int bus_state;

        bus_state = atomic_read(&chip_ddr_burst_interface->bus_state);
        bus_state &= ~DDR_END_BUS_BUSY;
        atomic_set(&chip_ddr_burst_interface->bus_state, bus_state);
    }
}

/****************************************************************************************************************
* Function Name :   chip_ddr_burst_engine_is_can_submit_move_data_from_ddr_to_dpram_service_req
* Description:      whether or not submit data transfer form ddr to dpram
* Input         :   param1 is chip ddr burst engine management struct
*                   param2 is dpram page node management strcut
* Output        :   null
* Return        :   1 is meaning ok, other meaning can't
****************************************************************************************************************/
static int  chip_ddr_burst_engine_is_can_submit_move_data_from_ddr_to_dpram_service_req(chip_ddr_burst_engine_t *chip_ddr_burst_interface, dpram_page_node_t **ptr_dpram_page)
{
    int ret = 0;
    if(chip_ddr_burst_interface != NULL){
        dpram_page_resource_manage_t *dpram_resource;
        int bus_state;

        bus_state = atomic_read(&chip_ddr_burst_interface->bus_state);
        if(!(bus_state&DDR_END_BUS_BUSY)){
			atomic_set(&chip_ddr_burst_interface->bus_state, (bus_state|DDR_END_BUS_BUSY));
            dpram_resource = &chip_ddr_burst_interface->dpram_resource;
            dpram_resource->op->get_dpram_page(dpram_resource, ptr_dpram_page);
			ret = 1;
        }
    }
    return ret;
}

/****************************************************************************************************************
* Function Name :   chip_ddr_burst_engine_notify_end_move_data_from_dpram_to_host_service_req
* Description:      notify the transfer from dpram to host is over
* Input         :   param1 is chip ddr burst engine management struct
*                   param2 is dpram page node management strcut
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void chip_ddr_burst_engine_notify_end_move_data_from_dpram_to_host_service_req(chip_ddr_burst_engine_t *chip_ddr_burst_interface, dpram_page_node_t **ptr_dpram_page)
{
    if((chip_ddr_burst_interface!=NULL) && (ptr_dpram_page!=NULL)){
        dpram_request_queue_t       *dpram_request_queue;

		if(chip_ddr_burst_interface->op != NULL){
			chip_ddr_burst_interface->op->release_end_move_data_from_dpram_to_host_service_req(chip_ddr_burst_interface, ptr_dpram_page);
		}
        dpram_request_queue = &chip_ddr_burst_interface->dpram_request_queue;
		dpram_request_queue->op->release_curr_consumer(dpram_request_queue);
		dpram_request_queue->op->trigger_chip_pending_service_request(dpram_request_queue);
    }
}

/****************************************************************************************************************
* Function Name :   chip_ddr_burst_engine_release_end_move_data_from_dpram_to_host_service_req
* Description:      release the transfer from dpram to host is over
* Input         :   param1 is chip ddr burst engine management struct
*                   param2 is dpram page node management strcut
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void chip_ddr_burst_engine_release_end_move_data_from_dpram_to_host_service_req(chip_ddr_burst_engine_t *chip_ddr_burst_interface, dpram_page_node_t **ptr_dpram_page)
{
    if((chip_ddr_burst_interface!=NULL) && (ptr_dpram_page!=NULL)){
        int bus_state;

        bus_state = atomic_read(&chip_ddr_burst_interface->bus_state);
        bus_state &= ~DDR_END_BUS_BUSY;
        atomic_set(&chip_ddr_burst_interface->bus_state, bus_state);
    }
}

/****************************************************************************************************************
* Function Name :   chip_ddr_burst_engine_submit_write_video_encode_osd_data_req
* Description:      submit osd's data transfer request
* Input         :   param1 is chip ddr burst engine management struct
*                   param2 is osd rectangle management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void chip_ddr_burst_engine_submit_write_video_encode_osd_data_req(chip_ddr_burst_engine_t *chip_ddr_burst_interface, osd_rectangle_entry_t *rectangle)
{
    if((chip_ddr_burst_interface!=NULL) && (rectangle!=NULL)){
        dpram_request_queue_t       *dpram_request_queue;
        osd_rectangle_dpram_tcb_t   *osd_dpram_req;

        dpram_request_queue = &chip_ddr_burst_interface->dpram_request_queue;
        osd_dpram_req = &rectangle->osd_dpram_req;
        dpram_request_queue->op->put_service_request_into_queue(dpram_request_queue, &osd_dpram_req->osd_data_req);
    }
}

/****************************************************************************************************************
* Function Name :   chip_ddr_burst_engine_ack_write_video_encode_osd_data_req
* Description:      ack osd's data transfer request
* Input         :   param1 is chip ddr burst engine management struct
*                   param2 is osd rectangle management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void chip_ddr_burst_engine_ack_write_video_encode_osd_data_req(chip_ddr_burst_engine_t *chip_ddr_burst_interface, osd_rectangle_entry_t *rectangle)
{
    if((chip_ddr_burst_interface!=NULL) && (rectangle!=NULL)){
        osd_rectangle_dpram_tcb_t   *osd_dpram_req;

        osd_dpram_req = &rectangle->osd_dpram_req;
        chip_ddr_burst_interface->op->notify_end_move_data_from_dpram_to_ddr_service_req(chip_ddr_burst_interface, &osd_dpram_req->dpram_page);
    }
}

/****************************************************************************************************************
* Function Name :   chip_ddr_burst_engine_delete_wirte_video_encode_osd_data_req
* Description:      delete osd's data transfer request
* Input         :   param1 is chip ddr burst engine management struct
*                   param2 is osd rectangle management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static int  chip_ddr_burst_engine_delete_wirte_video_encode_osd_data_req(chip_ddr_burst_engine_t *chip_ddr_burst_interface, osd_rectangle_entry_t *rectangle)
{
    int ret = 0;
    if((chip_ddr_burst_interface!=NULL) && (rectangle!=NULL)){
        dpram_request_queue_t       *dpram_request_queue;
        osd_rectangle_dpram_tcb_t   *osd_dpram_req;

        dpram_request_queue = &chip_ddr_burst_interface->dpram_request_queue;
        osd_dpram_req = &rectangle->osd_dpram_req;
        ret = dpram_request_queue->op->delete_service_request_from_queue(dpram_request_queue, &osd_dpram_req->osd_data_req);
    }
    return ret;
}

/****************************************************************************************************************
* Function Name :   chip_ddr_burst_engine_submit_write_video_encode_osd_attr_req
* Description:      submit osd's attr data transfer request
* Input         :   param1 is chip ddr burst engine management struct
*                   param2 is osd attr group management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void chip_ddr_burst_engine_submit_write_video_encode_osd_attr_req(chip_ddr_burst_engine_t *chip_ddr_burst_interface, osd_attribute_regs_group_t *attr_group)
{
    if((chip_ddr_burst_interface!=NULL) && (attr_group!=NULL)){
        dpram_request_queue_t       *dpram_request_queue;
        osd_rectangle_dpram_tcb_t   *osd_dpram_req;

        dpram_request_queue = &chip_ddr_burst_interface->dpram_request_queue;
        osd_dpram_req = &attr_group->osd_dpram_req;
        dpram_request_queue->op->put_service_request_into_queue(dpram_request_queue, &osd_dpram_req->osd_data_req);
    }
}

/****************************************************************************************************************
* Function Name :   chip_ddr_burst_engine_ack_write_video_encode_osd_attr_req
* Description:      ack osd's attr data transfer request
* Input         :   param1 is chip ddr burst engine management struct
*                   param2 is osd attr group management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void chip_ddr_burst_engine_ack_write_video_encode_osd_attr_req(chip_ddr_burst_engine_t *chip_ddr_burst_interface, osd_attribute_regs_group_t *attr_group)
{
    if((chip_ddr_burst_interface!=NULL) && (attr_group!=NULL)){
        osd_rectangle_dpram_tcb_t   *osd_dpram_req;
        osd_chan_engine_t           *osd_chan_engine;

        osd_dpram_req = &attr_group->osd_dpram_req;
        osd_chan_engine = to_osd_chan_engine_with_osd_attribute_regs_group(attr_group);
        chip_ddr_burst_interface->op->notify_end_move_data_from_dpram_to_ddr_service_req(chip_ddr_burst_interface, &osd_dpram_req->dpram_page);
    }
}

/****************************************************************************************************************
* Function Name :   chip_ddr_burst_engine_delete_write_video_encode_osd_attr_req
* Description:      delete osd's attr data transfer request
* Input         :   param1 is chip ddr burst engine management struct
*                   param2 is osd attr group management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static int  chip_ddr_burst_engine_delete_write_video_encode_osd_attr_req(chip_ddr_burst_engine_t *chip_ddr_burst_interface, osd_attribute_regs_group_t *attr_group)
{
    int ret = 0;

    if((chip_ddr_burst_interface!=NULL) && (attr_group!=NULL)){
        dpram_request_queue_t       *dpram_request_queue;
        osd_rectangle_dpram_tcb_t   *osd_dpram_req;

        dpram_request_queue = &chip_ddr_burst_interface->dpram_request_queue;
        osd_dpram_req = &attr_group->osd_dpram_req;
        ret = dpram_request_queue->op->delete_service_request_from_queue(dpram_request_queue, &osd_dpram_req->osd_data_req);
    }
    return ret;
}

/****************************************************************************************************************
* Function Name :   chip_ddr_burst_engine_submit_read_audio_req
* Description:      submit read audio encode data transfer request
* Input         :   param1 is chip ddr burst engine management struct
*                   param2 is audio chan driver management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void chip_ddr_burst_engine_submit_read_audio_req(chip_ddr_burst_engine_t *chip_ddr_burst_interface, isil_audio_driver_t *audio_driver)
{
    if((chip_ddr_burst_interface!=NULL) && (audio_driver!=NULL)){
        dpram_request_queue_t       *dpram_request_queue;

        dpram_request_queue = &chip_ddr_burst_interface->dpram_request_queue;
        dpram_request_queue->op->put_service_request_into_queue(dpram_request_queue, &audio_driver->audio_bitstream_req);
    }
}

/****************************************************************************************************************
* Function Name :   chip_ddr_burst_engine_ack_read_audio_req
* Description:      ack read audio encode data transfer request
* Input         :   param1 is chip ddr burst engine management struct
*                   param2 is audio chan driver management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void chip_ddr_burst_engine_ack_read_audio_req(chip_ddr_burst_engine_t *chip_ddr_burst_interface, isil_audio_driver_t *audio_driver)
{
    if((chip_ddr_burst_interface!=NULL) && (audio_driver!=NULL)){
        chip_ddr_burst_interface->op->notify_end_move_data_from_dpram_to_host_service_req(chip_ddr_burst_interface, &audio_driver->dpram_page);
    }
}

/****************************************************************************************************************
* Function Name :   chip_ddr_burst_engine_delete_read_audio_req
* Description:      delete read audio encode data transfer request
* Input         :   param1 is chip ddr burst engine management struct
*                   param2 is audio chan driver management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static int  chip_ddr_burst_engine_delete_read_audio_req(chip_ddr_burst_engine_t *chip_ddr_burst_interface, isil_audio_driver_t *audio_driver)
{
    int ret = 0;

    if((chip_ddr_burst_interface!=NULL) && (audio_driver!=NULL)){
        dpram_request_queue_t       *dpram_request_queue;

        dpram_request_queue = &chip_ddr_burst_interface->dpram_request_queue;
        ret = dpram_request_queue->op->delete_service_request_from_queue(dpram_request_queue, &audio_driver->audio_bitstream_req);
    }   
    return ret;
}


/****************************************************************************************************************
* Function Name :   chip_ddr_burst_engine_submit_write_audio_req
* Description:      submit write audio decode data transfer request
* Input         :   param1 is chip ddr burst engine management struct
*                   param2 is audio chan driver management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void chip_ddr_burst_engine_submit_write_audio_req(chip_ddr_burst_engine_t *chip_ddr_burst_interface, isil_audio_driver_t *audio_driver)
{
    if((chip_ddr_burst_interface!=NULL) && (audio_driver!=NULL)){
        dpram_request_queue_t       *dpram_request_queue;

        dpram_request_queue = &chip_ddr_burst_interface->dpram_request_queue;
        dpram_request_queue->op->put_service_request_into_queue(dpram_request_queue, &audio_driver->audio_bitstream_req);
    }
}

/****************************************************************************************************************
* Function Name :   chip_ddr_burst_engine_ack_write_audio_req
* Description:      ack write audio decode data transfer request
* Input         :   param1 is chip ddr burst engine management struct
*                   param2 is audio chan driver management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void chip_ddr_burst_engine_ack_write_audio_req(chip_ddr_burst_engine_t *chip_ddr_burst_interface, isil_audio_driver_t *audio_driver)
{
    if((chip_ddr_burst_interface!=NULL) && (audio_driver!=NULL)){
        chip_ddr_burst_interface->op->notify_end_move_data_from_dpram_to_ddr_service_req(chip_ddr_burst_interface, &audio_driver->dpram_page);
    }
}

/****************************************************************************************************************
* Function Name :   chip_ddr_burst_engine_delete_write_audio_req
* Description:      delete write audio decode data transfer request
* Input         :   param1 is chip ddr burst engine management struct
*                   param2 is audio chan driver management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static int  chip_ddr_burst_engine_delete_write_audio_req(chip_ddr_burst_engine_t *chip_ddr_burst_interface, isil_audio_driver_t *audio_driver)
{
    int ret = 0;

    if((chip_ddr_burst_interface!=NULL) && (audio_driver!=NULL)){
        dpram_request_queue_t       *dpram_request_queue;

        dpram_request_queue = &chip_ddr_burst_interface->dpram_request_queue;
        ret = dpram_request_queue->op->delete_service_request_from_queue(dpram_request_queue, &audio_driver->audio_bitstream_req);
    }   
    return ret;
}

#if	defined(PCI_PLATFORM)
/****************************************************************************************************************
* Function Name :   chip_ddr_burst_engine_submit_read_mjpeg_req
* Description:      submit read mjpeg encode data transfer request
* Input         :   param1 is chip ddr burst engine management struct
*                   param2 is mjpeg encode chan management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void chip_ddr_burst_engine_submit_read_mjpeg_req(chip_ddr_burst_engine_t *chip_ddr_burst_interface, isil_jpeg_logic_encode_chan_t *jpeg_logic_chan)
{
    if((chip_ddr_burst_interface!=NULL) && (jpeg_logic_chan!=NULL)){
        dpram_request_queue_t       *dpram_request_queue;

        dpram_request_queue = &chip_ddr_burst_interface->dpram_request_queue;
        dpram_request_queue->op->put_service_request_into_queue(dpram_request_queue, &jpeg_logic_chan->read_video_bitstream_req);
    }
}

/****************************************************************************************************************
* Function Name :   chip_ddr_burst_engine_ack_read_mjpeg_req
* Description:      ack read mjpeg encode data transfer request
* Input         :   param1 is chip ddr burst engine management struct
*                   param2 is mjpeg encode chan management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void chip_ddr_burst_engine_ack_read_mjpeg_req(chip_ddr_burst_engine_t *chip_ddr_burst_interface, isil_jpeg_logic_encode_chan_t *jpeg_logic_chan)
{
    if((chip_ddr_burst_interface!=NULL) && (jpeg_logic_chan!=NULL)){
        chip_ddr_burst_interface->op->notify_end_move_data_from_dpram_to_host_service_req(chip_ddr_burst_interface, &jpeg_logic_chan->dpram_page);
    }
}

/****************************************************************************************************************
* Function Name :   chip_ddr_burst_engine_delete_read_mjpeg_req
* Description:      delete read mjpeg encode data transfer request
* Input         :   param1 is chip ddr burst engine management struct
*                   param2 is mjpeg encode chan management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static int  chip_ddr_burst_engine_delete_read_mjpeg_req(chip_ddr_burst_engine_t *chip_ddr_burst_interface, isil_jpeg_logic_encode_chan_t *jpeg_logic_chan)
{
    int ret = 0;

    if((chip_ddr_burst_interface!=NULL) && (jpeg_logic_chan!=NULL)){
        dpram_request_queue_t       *dpram_request_queue;

        dpram_request_queue = &chip_ddr_burst_interface->dpram_request_queue;
        dpram_request_queue->op->delete_service_request_from_queue(dpram_request_queue, &jpeg_logic_chan->read_video_bitstream_req);
    }
    return ret;
}
#endif

/****************************************************************************************************************
* Veriable Name :   chip_ddr_burst_engine_op
* Description   :   for op chip ddr burst engine, we define general interface
****************************************************************************************************************/
static struct chip_ddr_burst_engine_operation   chip_ddr_burst_engine_op = {
    .init = chip_ddr_burst_engine_init,
	.get_dpram_request_queue = chip_ddr_burst_engine_get_dpram_request_queue,
    .get_ddr_burst_interface = chip_ddr_burst_engine_get_ddr_burst_interface,
    .get_dpram_page_resource_manage = chip_ddr_burst_engine_get_dpram_page_resource_manage,

    .is_can_submit_move_data_from_host_to_dpram_service_req = chip_ddr_burst_engine_is_can_submit_move_data_from_host_to_dpram_service_req,
    .notify_end_move_data_from_dpram_to_ddr_service_req = chip_ddr_burst_engine_notify_end_move_data_from_dpram_to_ddr_service_req,
    .release_end_move_data_from_dpram_to_ddr_service_req = chip_ddr_burst_engine_release_end_move_data_from_dpram_to_ddr_service_req,

    .is_can_submit_move_data_from_ddr_to_dpram_service_req = chip_ddr_burst_engine_is_can_submit_move_data_from_ddr_to_dpram_service_req,
    .notify_end_move_data_from_dpram_to_host_service_req = chip_ddr_burst_engine_notify_end_move_data_from_dpram_to_host_service_req,
    .release_end_move_data_from_dpram_to_host_service_req = chip_ddr_burst_engine_release_end_move_data_from_dpram_to_host_service_req,

    .submit_write_video_encode_osd_data_req = chip_ddr_burst_engine_submit_write_video_encode_osd_data_req,
    .ack_write_video_encode_osd_data_req = chip_ddr_burst_engine_ack_write_video_encode_osd_data_req,
    .delete_wirte_video_encode_osd_data_req = chip_ddr_burst_engine_delete_wirte_video_encode_osd_data_req,

    .submit_write_video_encode_osd_attr_req = chip_ddr_burst_engine_submit_write_video_encode_osd_attr_req,
    .ack_write_video_encode_osd_attr_req = chip_ddr_burst_engine_ack_write_video_encode_osd_attr_req,
    .delete_write_video_encode_osd_attr_req = chip_ddr_burst_engine_delete_write_video_encode_osd_attr_req,

    .submit_read_audio_req = chip_ddr_burst_engine_submit_read_audio_req,
    .ack_read_audio_req = chip_ddr_burst_engine_ack_read_audio_req,
    .delete_read_audio_req = chip_ddr_burst_engine_delete_read_audio_req,

    .submit_write_audio_req = chip_ddr_burst_engine_submit_write_audio_req,
    .ack_write_audio_req = chip_ddr_burst_engine_ack_write_audio_req,
    .delete_write_audio_req = chip_ddr_burst_engine_delete_write_audio_req,

#if	defined(PCI_PLATFORM)
    .submit_read_mjpeg_req = chip_ddr_burst_engine_submit_read_mjpeg_req,
    .ack_read_mjpeg_req = chip_ddr_burst_engine_ack_read_mjpeg_req,
    .delete_read_mjpeg_req = chip_ddr_burst_engine_delete_read_mjpeg_req,
#endif
};

/****************************************************************************************************************
* Function Name :   init_chip_ddr_burst_engine
* Description:      init chip ddr burst engine
* Input         :   param1 is chip ddr burst engine management struct
*                   param2 is chip management struct
* Output        :   null
* Return        :   null
****************************************************************************************************************/
void    init_chip_ddr_burst_engine(chip_ddr_burst_engine_t *chip_ddr_burst_interface, isil_chip_t *chip)
{
    if((chip_ddr_burst_interface!=NULL) && (chip!=NULL)){
        chip_ddr_burst_interface->op = &chip_ddr_burst_engine_op;
        chip_ddr_burst_interface->op->init(chip_ddr_burst_interface, chip);
    }
}


