/****************************************************************************************************************
* File Name         : isil_h264_video_buf.c
* Author            : shangchang Tang
* Revision          : 1.2.1.1275
* Date              : 2012-03-29
* Description       : this file focus on video data's buf pool manage, and video queue manage.
* Change log        :
*
****************************************************************************************************************
* THE PRESENT SOFTWARE IS DISTRIBUTED IN THE HOPE THAT IT WILL PROVIDE CUSTOMERS
* WITH CODING INFORMATION TO SPEED UP THEIR PRODUCT DEVELOPMENT, BUT WITHOUT
* ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS
* FOR A PARTICULAR PURPOSE. AS A RESULT, TECHWELL SHALL NOT BE HELD LIABLE FOR ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
****************************************************************************************************************/
#include	<isil5864/isil_common.h>


/****************************************************************************************************************
* Function Name :   isil_video_packet_tcb_init
* Description   :   video packet tcb's init.
* Input         :   param1 is video packet tcb struct, 
*                   param2 is video buf pool struct,
*                   param3 is id
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static int  isil_video_packet_tcb_init(isil_video_packet_tcb_t *video_packet_tcb, isil_video_chan_buf_pool_t *video_chan_buf_pool, int id)
{
    if((video_packet_tcb!=NULL) && (video_chan_buf_pool!=NULL)){
        tcb_node_t  *node = &video_packet_tcb->packet_node;
        spin_lock_init(&video_packet_tcb->lock);
        atomic_set(&video_packet_tcb->ref_count, 0);
        node->op = &tcb_node_op;
        node->op->init(node);
        node->op->set_priv(node, video_packet_tcb);
        video_packet_tcb->packet_id = id;
        video_packet_tcb->packet_size = video_chan_buf_pool->video_packet_buffer_size;
        video_packet_tcb->payload_len = 0;
        video_packet_tcb->consumer_len = 0;
        video_packet_tcb->data = (u8*)__get_free_page(GFP_KERNEL);
        if(video_packet_tcb->data == NULL) {
            ISIL_DBG(ISIL_DBG_ERR, "no memory\n");
            return -EINVAL;
        }
        video_packet_tcb->dma_addr = 0;
        video_packet_tcb->id_of_packet_queue = 0;

        return 0;
    }

    return -EINVAL;
}

/****************************************************************************************************************
* Function Name :   isil_video_packet_tcb_reset
* Description   :   video packet tcb's reset.
* Input         :   param1 is video packet tcb struct, 
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static int  isil_video_packet_tcb_reset(isil_video_packet_tcb_t *video_packet_tcb)
{
    if(video_packet_tcb!=NULL){
        atomic_set(&video_packet_tcb->ref_count, 0);
        video_packet_tcb->payload_len = 0;
        video_packet_tcb->consumer_len = 0;
        video_packet_tcb->dma_addr = 0;
        video_packet_tcb->id_of_packet_queue = 0;

        return 0;
    }

    return -EINVAL;
}

/****************************************************************************************************************
* Function Name :   isil_video_packet_tcb_release
* Description   :   video packet tcb's release.
* Input         :   param1 is video packet tcb struct, 
*                   param2 is video buf pool struct,
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static int  isil_video_packet_tcb_release(isil_video_packet_tcb_t **ptr_video_packet_tcb, isil_video_chan_buf_pool_t *video_chan_buf_pool)
{
    if((*ptr_video_packet_tcb!=NULL) && (video_chan_buf_pool!=NULL)){
        isil_video_packet_tcb_t	*video_packet_tcb = *ptr_video_packet_tcb;
        unsigned long	flags;

        spin_lock_irqsave(&video_packet_tcb->lock, flags);
        if(atomic_read(&video_packet_tcb->ref_count) > 1){
            atomic_dec(&video_packet_tcb->ref_count);
        } else {
            tcb_node_t	*node;
            if(video_packet_tcb->op != NULL){
                video_packet_tcb->op->reset(video_packet_tcb);
            }
            node = &video_packet_tcb->packet_node;
            if(node->op != NULL){
                node->op->release(node, &video_chan_buf_pool->video_packet_pool_tcb);
            }
        }
        *ptr_video_packet_tcb = NULL;
        spin_unlock_irqrestore(&video_packet_tcb->lock, flags);

        return 0;
    }

    return -EINVAL;
}

/****************************************************************************************************************
* Function Name :   isil_video_packet_tcb_reference
* Description   :   video packet tcb's reference.
* Input         :   param1 is src video packet tcb struct, 
*                   param2 is dest video packet tcb struct,
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void isil_video_packet_tcb_reference(isil_video_packet_tcb_t *src_video_packet_tcb, isil_video_packet_tcb_t **ptr_dest_video_packet_tcb)
{
    if((src_video_packet_tcb!=NULL) && (ptr_dest_video_packet_tcb!=NULL)){
        unsigned long	flags;
        spin_lock_irqsave(&src_video_packet_tcb->lock, flags);
        atomic_inc(&src_video_packet_tcb->ref_count);
        *ptr_dest_video_packet_tcb = src_video_packet_tcb;
        spin_unlock_irqrestore(&src_video_packet_tcb->lock, flags);
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_packet_tcb_dma_map
* Description   :   get addr for dma transfer.
* Input         :   param1 is src video packet tcb struct, 
*                   param2 is dest video packet tcb struct,
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void isil_video_packet_tcb_dma_map(isil_video_packet_tcb_t *video_packet_tcb, enum dma_data_direction direction)
{
    /*if(video_packet_tcb != NULL) {
        if(video_packet_tcb->data != NULL) {
            video_packet_tcb->dma_addr = dma_map_single(NULL, video_packet_tcb->data, video_packet_tcb->packet_size, direction);
            if (video_packet_tcb->dma_addr == 0){
                printk("%s.%d err\n", __FUNCTION__, __LINE__);
            }
        }
    }*/
}

/****************************************************************************************************************
* Function Name :   isil_video_packet_tcb_dma_unmap
* Description   :   release dma addr.
* Input         :   param1 is src video packet tcb struct, 
*                   param2 is dest video packet tcb struct,
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void isil_video_packet_tcb_dma_unmap(isil_video_packet_tcb_t *video_packet_tcb, enum dma_data_direction direction)
{
    /*if(video_packet_tcb != NULL) {
        if (video_packet_tcb->dma_addr != 0) {
            dma_unmap_single(NULL, video_packet_tcb->dma_addr, video_packet_tcb->packet_size, direction);
            video_packet_tcb->dma_addr = 0;
        }
    }*/
}

/****************************************************************************************************************
* Function Name :   isil_video_packet_tcb_submit
* Description   :   sumbit packet data.
* Input         :   param1 is video packet tcb struct, 
*                   param2 is user space buf
*                   param3 is size
*                   param4 is position
*                   param5 is nal
* Output        :   null
* Return        :   real count
****************************************************************************************************************/
static size_t   isil_video_packet_tcb_submit(isil_video_packet_tcb_t *packet, char __user *data, size_t count, loff_t *ppos, h264_nal_t *nal)
{
    loff_t  buf_offset = 0;
    if((packet) && (packet->payload_len<=PAGE_SIZE)){
        u8  *src_ptr;
        int transfer_len;
        transfer_len = H264_MIN(packet->packet_size, packet->payload_len);
        transfer_len = H264_MIN((transfer_len-packet->consumer_len), packet->payload_len);
        transfer_len = H264_MIN(transfer_len, count);
        src_ptr = &packet->data[packet->consumer_len];
        if (copy_to_user(data, src_ptr, transfer_len) != 0) {
		ISIL_DBG(ISIL_DBG_INFO, "failed to copy from user\n");
	}
        data += transfer_len;
        packet->consumer_len += transfer_len;
        buf_offset += transfer_len;
    } else {
	    ISIL_DBG(ISIL_DBG_INFO, ":  BAD copy parameter is wrong!!%s.%d, count = %08x, payload_len = %08x\n\n", __FUNCTION__, __LINE__, 
		    (unsigned int) count, packet->payload_len);
    }
    *ppos += buf_offset;
    return (size_t)buf_offset;
}


/****************************************************************************************************************
* Function Name :   isil_video_packet_tcb_slicent_discard
* Description   :   discard packet
* Input         :   param1 is video packet tcb struct, 
*                   param2 is user space buf
*                   param3 is position
*                   param4 is nal
* Output        :   null
* Return        :   real count
****************************************************************************************************************/
static size_t  isil_video_packet_tcb_slicent_discard(isil_video_packet_tcb_t *packet, char __user *data, size_t count, loff_t *ppos, h264_nal_t *nal)
{
    loff_t  buf_offset = 0;
    if((packet != NULL) && (count>=packet->payload_len) && (packet->payload_len <= packet->packet_size)){
        buf_offset = packet->payload_len;
    } else {
        ISIL_DBG(ISIL_DBG_INFO, ":  BAD copy parameter is wrong!!%s.%d, count = %08x, payload_len = %08x\n\n", __FUNCTION__, __LINE__, 
                (unsigned int) count, packet->payload_len);
    }
    *ppos += buf_offset;
    return (size_t)buf_offset;

}

/****************************************************************************************************************
* Function Name :   isil_video_packet_tcb_copy
* Description   :   copy packet
* Input         :   param1 is src video packet tcb struct, 
*                   param2 is dest video packet tcb struct
* Output        :   null
* Return        :   real count
****************************************************************************************************************/
static int isil_video_packet_tcb_copy(isil_video_packet_tcb_t *src, isil_video_packet_tcb_t *dst)
{
    if(src && dst) {
        dst->consumer_len = src->consumer_len;
        dst->payload_len  = src->payload_len;
        memcpy(dst->data, src->data, src->payload_len);

        return ISIL_OK;
    }

    return ISIL_ERR;
}

static	struct isil_video_packet_operation	isil_video_packet_tcb_op = {
    .init = isil_video_packet_tcb_init,
    .reset = isil_video_packet_tcb_reset,
    .release = isil_video_packet_tcb_release,
    .reference = isil_video_packet_tcb_reference,

    .dma_map = isil_video_packet_tcb_dma_map,
    .dma_unmap = isil_video_packet_tcb_dma_unmap,

    .submit = isil_video_packet_tcb_submit,
    .slicent_discard = isil_video_packet_tcb_slicent_discard,
    .copy   = isil_video_packet_tcb_copy,
};

/****************************************************************************************************************
* Function Name :   isil_video_packet_queue_get
* Description   :   get a packet from queue
* Input         :   param1 is video packet queue 
* Output        :   get video packet
* Return        :   null
****************************************************************************************************************/
static void isil_video_packet_queue_get(isil_video_packet_tcb_queue_t *video_packet_queue, isil_video_packet_tcb_t **ptr_video_packet_tcb)
{
    if((video_packet_queue!=NULL) && (ptr_video_packet_tcb!=NULL)){
        tcb_node_queue_t    *queue;
        *ptr_video_packet_tcb = NULL;
        queue = &video_packet_queue->queue_node;
        if(queue->op != NULL){
            tcb_node_t  *temp_node;
            queue->op->get(queue, &temp_node);
            if(temp_node != NULL){
                *ptr_video_packet_tcb = to_video_packet_buf_tcb(temp_node);
            }
        }
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_packet_queue_try_get
* Description   :   try get a packet from queue
* Input         :   param1 is video packet queue 
* Output        :   get video packet
* Return        :   null
****************************************************************************************************************/
static void isil_video_packet_queue_try_get(isil_video_packet_tcb_queue_t *video_packet_queue, isil_video_packet_tcb_t **ptr_video_packet_tcb)
{
    if((video_packet_queue!=NULL) && (ptr_video_packet_tcb!=NULL)){
        tcb_node_queue_t	*queue;
        *ptr_video_packet_tcb = NULL;
        queue = &video_packet_queue->queue_node;
        if(queue->op != NULL){
            tcb_node_t  *temp_node;
            queue->op->try_get(queue, &temp_node);
            if(temp_node != NULL){
                *ptr_video_packet_tcb = to_video_packet_buf_tcb(temp_node);
            }
        }
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_packet_queue_get_tailer
* Description   :   get a packet from queue tail
* Input         :   param1 is video packet queue 
* Output        :   get video packet
* Return        :   null
****************************************************************************************************************/
static void isil_video_packet_queue_get_tailer(isil_video_packet_tcb_queue_t *video_packet_queue, isil_video_packet_tcb_t **ptr_video_packet_tcb)
{
    if((video_packet_queue!=NULL) && (ptr_video_packet_tcb!=NULL)){
        tcb_node_queue_t    *queue;
        *ptr_video_packet_tcb = NULL;
        queue = &video_packet_queue->queue_node;
        if(queue->op != NULL){
            tcb_node_t	*temp_node;
            queue->op->get_tailer(queue, &temp_node);
            if(temp_node != NULL){
                *ptr_video_packet_tcb = to_video_packet_buf_tcb(temp_node);
            }
        }
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_packet_queue_try_get_tailer
* Description   :   try get a packet from queue tail
* Input         :   param1 is video packet queue 
* Output        :   get video packet
* Return        :   null
****************************************************************************************************************/
static void isil_video_packet_queue_try_get_tailer(isil_video_packet_tcb_queue_t *video_packet_queue, isil_video_packet_tcb_t **ptr_video_packet_tcb)
{
    if((video_packet_queue!=NULL) && (ptr_video_packet_tcb!=NULL)){
        tcb_node_queue_t    *queue;
        *ptr_video_packet_tcb = NULL;
        queue = &video_packet_queue->queue_node;
        if(queue->op != NULL){
            tcb_node_t	*temp_node;
            queue->op->try_get_tailer(queue, &temp_node);
            if(temp_node != NULL){
                *ptr_video_packet_tcb = to_video_packet_buf_tcb(temp_node);
            }
        }
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_packet_queue_put
* Description   :   put a packet into queue
* Input         :   param1 is video packet queue 
*                   param2 is video packet
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void isil_video_packet_queue_put(isil_video_packet_tcb_queue_t *video_packet_queue, isil_video_packet_tcb_t *video_packet_tcb)
{
    if((video_packet_queue!=NULL) && (video_packet_tcb!=NULL)){
        tcb_node_queue_t    *queue;
        queue = &video_packet_queue->queue_node;
        if(queue->op != NULL){
            queue->op->put(queue, &video_packet_tcb->packet_node);
            video_packet_tcb->id_of_packet_queue = queue->op->get_queue_curr_entry_number(queue);
        }
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_packet_queue_put_header
* Description   :   put a packet into queue header
* Input         :   param1 is video packet queue 
*                   param2 is video packet
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void isil_video_packet_queue_put_header(isil_video_packet_tcb_queue_t *video_packet_queue, isil_video_packet_tcb_t *video_packet_tcb)
{
    if((video_packet_queue!=NULL) && (video_packet_tcb!=NULL)){
        tcb_node_queue_t    *queue;
        queue = &video_packet_queue->queue_node;
        if(queue->op != NULL){
            queue->op->put_header(queue, &video_packet_tcb->packet_node);
        }
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_packet_queue_get_curr_producer_from_pool
* Description   :   get a packet from buf pool
* Input         :   param1 is video packet queue 
*                   param2 is video packet buf pool
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void isil_video_packet_queue_get_curr_producer_from_pool(isil_video_packet_tcb_queue_t *video_packet_queue, isil_video_chan_buf_pool_t *video_chan_buf_pool)
{
    if((video_packet_queue!=NULL) && (video_chan_buf_pool!=NULL)){
        unsigned long	flags;
        spin_lock_irqsave(&video_packet_queue->lock, flags);
        if(video_packet_queue->curr_producer == NULL){
            spin_unlock_irqrestore(&video_packet_queue->lock, flags);
            if(video_chan_buf_pool->op != NULL){
                video_chan_buf_pool->op->get_video_packet_tcb(video_chan_buf_pool, &video_packet_queue->curr_producer);
            }
            /*if(video_packet_queue->curr_producer != NULL){
              video_packet_queue->curr_producer->op->dma_map(video_packet_queue->curr_producer, DMA_FROM_DEVICE);
              }*/
            spin_lock_irqsave(&video_packet_queue->lock, flags);
        }
        spin_unlock_irqrestore(&video_packet_queue->lock, flags);
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_packet_queue_try_get_curr_producer_from_pool
* Description   :   try get a packet from buf pool
* Input         :   param1 is video packet queue 
*                   param2 is video packet buf pool
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void isil_video_packet_queue_try_get_curr_producer_from_pool(isil_video_packet_tcb_queue_t *video_packet_queue, isil_video_chan_buf_pool_t *video_chan_buf_pool)
{
    if((video_packet_queue!=NULL) && (video_chan_buf_pool!=NULL)){
        unsigned long   flags;
        spin_lock_irqsave(&video_packet_queue->lock, flags);
        if(video_packet_queue->curr_producer == NULL) {
            if(video_chan_buf_pool->op != NULL){
                video_chan_buf_pool->op->try_get_video_packet_tcb(video_chan_buf_pool, &video_packet_queue->curr_producer);
            }
            /*if(video_packet_queue->curr_producer != NULL) {
              video_packet_queue->curr_producer->op->dma_map(video_packet_queue->curr_producer, DMA_FROM_DEVICE);
              }*/
        }
        spin_unlock_irqrestore(&video_packet_queue->lock, flags);
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_packet_queue_put_curr_producer_into_queue
* Description   :   put curr producer packet into queue
* Input         :   param1 is video packet queue 
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void isil_video_packet_queue_put_curr_producer_into_queue(isil_video_packet_tcb_queue_t *video_packet_queue)
{
    if(video_packet_queue != NULL){
        unsigned long	flags;
        spin_lock_irqsave(&video_packet_queue->lock, flags);
        if(video_packet_queue->curr_producer != NULL){
            if(video_packet_queue->op != NULL){
                video_packet_queue->op->put(video_packet_queue, video_packet_queue->curr_producer);
            } else {
                isil_video_packet_queue_put(video_packet_queue, video_packet_queue->curr_producer);
            }
            video_packet_queue->curr_producer = NULL;
        }
        spin_unlock_irqrestore(&video_packet_queue->lock, flags);
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_packet_queue_release_curr_producer
* Description   :   release curr producer packet
* Input         :   param1 is video packet queue
*                   param2 is video buf pool 
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void isil_video_packet_queue_release_curr_producer(isil_video_packet_tcb_queue_t *video_packet_queue, isil_video_chan_buf_pool_t *video_chan_buf_pool)
{
    if((video_packet_queue!=NULL) && (video_chan_buf_pool!=NULL)){
        unsigned long	flags;
        spin_lock_irqsave(&video_packet_queue->lock, flags);
        if(video_packet_queue->curr_producer != NULL){
            if(video_packet_queue->curr_producer->op != NULL){
                video_packet_queue->curr_producer->op->release(&video_packet_queue->curr_producer, video_chan_buf_pool);
            } else {
                isil_video_packet_tcb_release(&video_packet_queue->curr_producer, video_chan_buf_pool);
            }
        }
        spin_unlock_irqrestore(&video_packet_queue->lock, flags);
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_packet_queue_get_curr_consumer_from_queue
* Description   :   get curr consumer packet from queue
* Input         :   param1 is video packet queue
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void isil_video_packet_queue_get_curr_consumer_from_queue(isil_video_packet_tcb_queue_t *video_packet_queue)
{
    if(video_packet_queue != NULL){
        unsigned long	flags;
        spin_lock_irqsave(&video_packet_queue->lock, flags);
        if(video_packet_queue->curr_consumer == NULL){
            spin_unlock_irqrestore(&video_packet_queue->lock, flags);
            if(video_packet_queue->op != NULL){
                video_packet_queue->op->get(video_packet_queue, &video_packet_queue->curr_consumer);
            } else {
                isil_video_packet_queue_get(video_packet_queue, &video_packet_queue->curr_consumer);
            }
            spin_lock_irqsave(&video_packet_queue->lock, flags);
        }
        spin_unlock_irqrestore(&video_packet_queue->lock, flags);
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_packet_queue_try_get_curr_consumer_from_queue
* Description   :   try get curr consumer packet from queue
* Input         :   param1 is video packet queue
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void isil_video_packet_queue_try_get_curr_consumer_from_queue(isil_video_packet_tcb_queue_t *video_packet_queue)
{
    if(video_packet_queue != NULL){
        unsigned long	flags;
        spin_lock_irqsave(&video_packet_queue->lock, flags);
        if(video_packet_queue->curr_consumer == NULL){
            if(video_packet_queue->op != NULL){
                video_packet_queue->op->try_get(video_packet_queue, &video_packet_queue->curr_consumer);
            } else {
                isil_video_packet_queue_try_get(video_packet_queue, &video_packet_queue->curr_consumer);
            }
        }
        spin_unlock_irqrestore(&video_packet_queue->lock, flags);
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_packet_queue_release_curr_consumer
* Description   :   release curr consumer packet
* Input         :   param1 is video packet queue
*                   param2 is video buf pool
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void isil_video_packet_queue_release_curr_consumer(isil_video_packet_tcb_queue_t *video_packet_queue, isil_video_chan_buf_pool_t *video_chan_buf_pool)
{
    if((video_packet_queue!=NULL) && (video_chan_buf_pool!=NULL)){
        unsigned long	flags;
        spin_lock_irqsave(&video_packet_queue->lock, flags);
        if(video_packet_queue->curr_consumer != NULL){
            if(video_packet_queue->curr_consumer->op != NULL){
                video_packet_queue->curr_consumer->op->release(&video_packet_queue->curr_consumer, video_chan_buf_pool);
            } else {
                isil_video_packet_tcb_release(&video_packet_queue->curr_consumer, video_chan_buf_pool);
            }
        }
        spin_unlock_irqrestore(&video_packet_queue->lock, flags);
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_packet_queue_get_curr_queue_entry_number
* Description   :   get queue's entry number
* Input         :   param1 is video packet queue
* Output        :   null
* Return        :   real number
****************************************************************************************************************/
static int  isil_video_packet_queue_get_curr_queue_entry_number(isil_video_packet_tcb_queue_t *video_packet_queue)
{
    int entry_number = 0;
    tcb_node_queue_t    *queue;
    queue = &video_packet_queue->queue_node;
    if(queue->op != NULL){
        entry_number = queue->op->get_queue_curr_entry_number(queue);
    }
    return entry_number;
}

/****************************************************************************************************************
* Function Name :   isil_video_packet_queue_release
* Description   :   release packet queue
* Input         :   param1 is video packet queue
*                   param2 is video buf pool
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void isil_video_packet_queue_release(isil_video_packet_tcb_queue_t *video_packet_queue, isil_video_chan_buf_pool_t *video_chan_buf_pool)
{
    if((video_packet_queue!=NULL) && (video_chan_buf_pool!=NULL)){
        remove_isil_video_packet_tcb_queue(video_packet_queue, video_chan_buf_pool);
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_packet_queue_init
* Description   :   init packet queue
* Input         :   param1 is video packet queue
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void isil_video_packet_queue_init(isil_video_packet_tcb_queue_t *video_packet_queue)
{
    if(video_packet_queue != NULL){
        tcb_node_queue_t    *queue;
        queue = &video_packet_queue->queue_node;
        queue->op = &tcb_node_queue_op;
        queue->op->init(queue);
        video_packet_queue->curr_consumer = NULL;
        video_packet_queue->curr_producer = NULL;
        video_packet_queue->start_timestamp = 0;
        video_packet_queue->total_duration = 0;
        spin_lock_init(&video_packet_queue->lock);
    }
}

static struct isil_video_packet_queue_operation   isil_video_packet_queue_op = {
    .get = isil_video_packet_queue_get,
    .try_get = isil_video_packet_queue_try_get,
    .get_tailer = isil_video_packet_queue_get_tailer,
    .try_get_tailer = isil_video_packet_queue_try_get_tailer,
    .put = isil_video_packet_queue_put,
    .put_header = isil_video_packet_queue_put_header,

    .get_curr_producer_from_pool = isil_video_packet_queue_get_curr_producer_from_pool,
    .try_get_curr_producer_from_pool = isil_video_packet_queue_try_get_curr_producer_from_pool,
    .put_curr_producer_into_queue = isil_video_packet_queue_put_curr_producer_into_queue,
    .release_curr_producer =  isil_video_packet_queue_release_curr_producer,
    .get_curr_consumer_from_queue = isil_video_packet_queue_get_curr_consumer_from_queue,
    .try_get_curr_consumer_from_queue = isil_video_packet_queue_try_get_curr_consumer_from_queue,
    .release_curr_consumer = isil_video_packet_queue_release_curr_consumer,

    .get_curr_queue_entry_number = isil_video_packet_queue_get_curr_queue_entry_number,
    .release = isil_video_packet_queue_release,
    .init = isil_video_packet_queue_init,
};

/****************************************************************************************************************
* Function Name :   init_isil_video_packet_tcb_queue
* Description   :   init packet queue's root entrance
* Input         :   param1 is video packet queue
* Output        :   null
* Return        :   null
****************************************************************************************************************/
void    init_isil_video_packet_tcb_queue(isil_video_packet_tcb_queue_t *video_packet_queue)
{
    if(video_packet_queue != NULL){
        video_packet_queue->op = &isil_video_packet_queue_op;
        video_packet_queue->op->init(video_packet_queue);
    }
}

/****************************************************************************************************************
* Function Name :   remove_isil_video_packet_tcb_queue
* Description   :   remove packet queue's root entrance
* Input         :   param1 is video packet queue
* Output        :   null
* Return        :   null
****************************************************************************************************************/
void    remove_isil_video_packet_tcb_queue(isil_video_packet_tcb_queue_t *video_packet_queue, isil_video_chan_buf_pool_t *video_chan_buf_pool)
{
    if((video_packet_queue!=NULL) && (video_chan_buf_pool!=NULL)){
        if(video_packet_queue->op != NULL){
            video_packet_queue->op->release_curr_producer(video_packet_queue, video_chan_buf_pool);
            video_packet_queue->op->release_curr_consumer(video_packet_queue, video_chan_buf_pool);
            while(video_packet_queue->op->get_curr_queue_entry_number(video_packet_queue)){
                video_packet_queue->op->get_curr_consumer_from_queue(video_packet_queue);
                if(video_packet_queue->curr_consumer == NULL){
                    break;
                }
                video_packet_queue->op->release_curr_consumer(video_packet_queue, video_chan_buf_pool);
            }
        } else {
            isil_video_packet_queue_release_curr_producer(video_packet_queue, video_chan_buf_pool);
            isil_video_packet_queue_release_curr_consumer(video_packet_queue, video_chan_buf_pool);
            while(isil_video_packet_queue_get_curr_queue_entry_number(video_packet_queue)){
                isil_video_packet_queue_get_curr_consumer_from_queue(video_packet_queue);
                if(video_packet_queue->curr_consumer == NULL){
                    break;
                }
                isil_video_packet_queue_release_curr_consumer(video_packet_queue, video_chan_buf_pool);
            }
        }
    }
}

static void isil_video_frame_tcb_nal_init(h264_nal_t *nal)
{
    if(nal != NULL){
        memset(nal, 0, sizeof(h264_nal_t));
        nal->sps_payload = nal->pps_payload = nal->slice_payload = nal->slice_head_buf = NULL;
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_frame_tcb_init
* Description   :   init video frame tcb
* Input         :   param1 is video frame
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void isil_video_frame_tcb_init(isil_video_frame_tcb_t *frame)
{
    if(frame!=NULL){
        tcb_node_t	*node = &frame->frame_node;
        node->op = &tcb_node_op;
        node->op->init(node);
        node->op->set_priv(node, frame);

        spin_lock_init(&frame->lock);
        atomic_set(&frame->ref_count, 0);
        frame->frame_type = H264_FRAME_TYPE_IDR;
        isil_video_frame_tcb_nal_init(&frame->nal);
        frame->frame_len = 0;
        frame->consumer_frame_offset = 0;
        frame->timestamp = 0;
        frame->duration = 0;
        frame->i_init_qp = DEFAULT_QP;
        frame->i_curr_qp = 0;
        frame->i_mb_x = 0;
        frame->i_mb_y = 0;
        frame->i_log_gop_value = 0;
        frame->slice_type = NAL_SLICE_IDR;
        frame->fps = 0;
        frame->frame_number = 0;
        frame->frame_is_err = 0;
        init_isil_video_packet_tcb_queue(&frame->video_packet_queue);
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_frame_tcb_reset
* Description   :   reset video frame tcb
* Input         :   param1 is video frame
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void isil_video_frame_tcb_reset(isil_video_frame_tcb_t *frame)
{
    if(frame != NULL){
        atomic_set(&frame->ref_count, 0);
        frame->frame_type = H264_FRAME_TYPE_IDR;
        isil_video_frame_tcb_nal_init(&frame->nal);
        frame->frame_len = 0;
        frame->consumer_frame_offset = 0;
        frame->timestamp = 0;
        frame->duration = 0;
        frame->i_init_qp = DEFAULT_QP;
        frame->i_curr_qp = 0;
        frame->i_mb_x = 0;
        frame->i_mb_y = 0;
        frame->i_log_gop_value = 0;
        frame->slice_type = NAL_SLICE_IDR;
        frame->fps = 0;
        frame->frame_number = 0;
        frame->frame_is_err = 0;
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_frame_tcb_release
* Description   :   release video frame
* Input         :   param1 is video frame
*                   param2 is video buf pool
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void isil_video_frame_tcb_release(isil_video_frame_tcb_t **ptr_frame, isil_video_chan_buf_pool_t *video_chan_buf_pool)
{
    if(ptr_frame != NULL){
        isil_video_frame_tcb_t    *frame = *ptr_frame;
        unsigned long	flags;
        if(video_chan_buf_pool == NULL){
            video_chan_buf_pool = frame->video_chan_buf_pool;
        }
        spin_lock_irqsave(&frame->lock, flags);
        if(atomic_read(&frame->ref_count) > 1){
            atomic_dec(&frame->ref_count);
        } else {
            tcb_node_t	*node;
            if(frame->op != NULL){
                frame->op->reset(frame);
            }
            if(frame->video_packet_queue.op != NULL){
                frame->video_packet_queue.op->release(&frame->video_packet_queue, video_chan_buf_pool);
            } else {
                isil_video_packet_queue_release(&frame->video_packet_queue, video_chan_buf_pool);
            }
            node = &frame->frame_node;
            if(node->op != NULL){
                node->op->release(node, &video_chan_buf_pool->video_frame_pool_tcb);
            }
        }
        *ptr_frame = NULL;
        spin_unlock_irqrestore(&frame->lock, flags);
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_frame_tcb_reference
* Description   :   reference video frame
* Input         :   param1 is src video frame
*                   param2 is dest video frame
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void isil_video_frame_tcb_reference(isil_video_frame_tcb_t *src_frame, isil_video_frame_tcb_t **ptr_dest_frame)
{
    if((src_frame!=NULL) && (ptr_dest_frame!=NULL)){
        unsigned long	flags;
        spin_lock_irqsave(&src_frame->lock, flags);
        atomic_inc(&src_frame->ref_count);
        *ptr_dest_frame = src_frame;
        spin_unlock_irqrestore(&src_frame->lock, flags);
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_frame_tcb_submit
* Description   :   video frame data submit
* Input         :   param1 is src video frame
*                   ...
* Output        :   null
* Return        :   null
****************************************************************************************************************/
#ifndef  NEW_HEADER
static size_t	isil_video_frame_tcb_submit(isil_video_frame_tcb_t *frame, char __user *data, size_t count, loff_t *ppos, int masterOrSubFlag, isil_video_chan_buf_pool_t *video_chan_buf_pool)
{
    isil_frame_header_t __user    *frame_header_ptr;
    isil_frame_header_t           isil_frame_head;
    isil_h264_idr_frame_pad_t __user  *idr_pad_ptr;
    isil_h264_idr_frame_pad_t     idr_frame_pad;
    isil_video_packet_tcb_queue_t *video_packet_queue;
    h264_nal_t  *nal;
    size_t  buf_size, packet_size;
    loff_t  buf_offset, packet_offset;
    unsigned int expect_len = sizeof(isil_frame_header_t) + sizeof(isil_h264_idr_frame_pad_t) + frame->frame_len;

    buf_size = count;
    buf_offset = 0;
    expect_len = tc_align_up(expect_len,4);

    if(frame != NULL){
        nal = &frame->nal;
        frame_header_ptr = (isil_frame_header_t*)data;
        buf_size -= sizeof(isil_frame_header_t);
        buf_offset += sizeof(isil_frame_header_t);
        memset(&isil_frame_head, 0, sizeof(isil_frame_header_t));
        isil_frame_head.codecType = ISIL_VIDEO_H264_CODEC_TYPE;
        isil_frame_head.streamType = masterOrSubFlag;
        isil_frame_head.frameSerial = frame->frame_number;
        isil_frame_head.frameType = frame->frame_type;
        isil_frame_head.timeStamp = frame->timestamp;
        isil_frame_head.payload_offset = sizeof(isil_frame_header_t);
        isil_frame_head.payloadLen = 0;
        switch(frame->frame_type) {
            case H264_FRAME_TYPE_IDR:
                if(count >= expect_len){
                    idr_pad_ptr = (isil_h264_idr_frame_pad_t*)frame_header_ptr->pad;
                    buf_size -= sizeof(isil_h264_idr_frame_pad_t);
                    buf_offset += sizeof(isil_h264_idr_frame_pad_t);
                    isil_frame_head.payloadLen += sizeof(isil_h264_idr_frame_pad_t);

                    memset(&idr_frame_pad, 0, sizeof(isil_h264_idr_frame_pad_t));
                    idr_frame_pad.fps = frame->fps;
                    idr_frame_pad.init_qp = frame->i_init_qp;
                    idr_frame_pad.log2MaxFrameNumNinus4 = frame->i_log_gop_value;
                    idr_frame_pad.mb_width_minus1 = frame->i_mb_x;
                    idr_frame_pad.mb_height_minus1 = frame->i_mb_y;
                    idr_frame_pad.i_curr_qp = frame->i_curr_qp;

                    idr_frame_pad.sps_frame_offset = sizeof(isil_h264_idr_frame_pad_t);
                    idr_frame_pad.sps_frame_size = nal->sps_paysize;
                    idr_frame_pad.pps_frame_offset = idr_frame_pad.sps_frame_offset + nal->sps_paysize;
                    idr_frame_pad.pps_frame_size = nal->pps_paysize;
                    idr_frame_pad.idr_frame_offset = idr_frame_pad.pps_frame_offset + nal->pps_paysize;
                    idr_frame_pad.idr_frame_size = 0;

                    video_packet_queue = &frame->video_packet_queue;
                    while(video_packet_queue->op->get_curr_queue_entry_number(video_packet_queue)){
                        video_packet_queue->op->get_curr_consumer_from_queue(video_packet_queue);
                        packet_size = video_packet_queue->curr_consumer->op->submit(video_packet_queue->curr_consumer, &data[buf_offset], buf_size, &packet_offset, nal);
                        video_packet_queue->op->release_curr_consumer(video_packet_queue, video_chan_buf_pool);
                        buf_size -= packet_size;
                        buf_offset += packet_size;
                        isil_frame_head.payloadLen += packet_size;
                    }
                    while(buf_offset & 3){
                        __put_user(0, &data[buf_offset]);
                        buf_size--;
                        buf_offset++;
                        idr_frame_pad.idr_frame_size++;
                        isil_frame_head.payloadLen++;
                    }
                    idr_frame_pad.idr_frame_size += frame->frame_len;
                    if(copy_to_user(frame_header_ptr->pad, (char*)&idr_frame_pad, sizeof(isil_h264_idr_frame_pad_t)) != 0){
                        ISIL_DBG(ISIL_DBG_INFO, ": copy_to_user failed\n");
                    }
                } else {
                    buf_offset = 0;
                }
                break;
            case H264_FRAME_TYPE_I:
            case H264_FRAME_TYPE_P:
                if(count >= (sizeof(isil_frame_header_t)+sizeof(isil_h264_idr_frame_pad_t)+frame->frame_len)){
                    video_packet_queue = &frame->video_packet_queue;
                    while(video_packet_queue->op->get_curr_queue_entry_number(video_packet_queue)){
                        video_packet_queue->op->get_curr_consumer_from_queue(video_packet_queue);
                        packet_size = video_packet_queue->curr_consumer->op->submit(video_packet_queue->curr_consumer, &data[buf_offset], buf_size, &packet_offset, nal);
                        video_packet_queue->op->release_curr_consumer(video_packet_queue, video_chan_buf_pool);
                        buf_size -= packet_size;
                        buf_offset += packet_size;
                        isil_frame_head.payloadLen += packet_size;
                    }
                    while(buf_offset & 3){
                        __put_user(0, &data[buf_offset]);
                        buf_size--;
                        buf_offset++;
                        isil_frame_head.payloadLen++;
                    }
                } else {
                    buf_offset = 0;
                }
                break;
            default:
                ISIL_DBG(ISIL_DBG_INFO, ": err 264 stream type\n", __FUNCTION__, __LINE__);
                break;
        }
        if(buf_offset){
            if(copy_to_user(frame_header_ptr, &isil_frame_head, sizeof(isil_frame_header_t)) != 0){
                ISIL_DBG(ISIL_DBG_INFO, ": copy_to_user failed\n");
            }
        }
    }
    *ppos += buf_offset;
    return (size_t)buf_offset;
}
#else

static size_t	isil_video_frame_tcb_submit(isil_video_frame_tcb_t *frame, char __user *data, size_t count, loff_t *ppos, int masterOrSubFlag, isil_video_chan_buf_pool_t *video_chan_buf_pool)
{
    struct isil_common_stream_header  icsh, *picsh;
    isil_transparent_msg_header_t  mh, *pmh;
    isil_h264_encode_msg_frame_type_t  emft_sps = {0}, emft_pps = {0}, emft_slice = {0}, *pemft;
    isil_frame_msg_t  fm_sps = {0}, fm_pps = {0}, fm_slice = {0}, *pfm;
    isil_video_packet_tcb_queue_t  *video_packet_queue;
    h264_nal_t  *nal;
    size_t  buf_size, packet_size;
    loff_t  buf_offset, packet_offset;
    char *pc, *crc_calc_base;
    char *crc_buf;
    unsigned int expect_len;// frame->frame_len is not include sps_paysize and pps_paysize
    struct isil_h264_logic_encode_chan *encode_chan;

    if(!data || !ppos || !video_chan_buf_pool)
        goto exception;

    encode_chan = container_of(video_chan_buf_pool,struct isil_h264_logic_encode_chan, encode_chan_buf_pool);
    buf_size = count;
    buf_offset = 0;
    picsh = &icsh;
    pmh = &mh;

    if(frame){
        if(frame->frame_type == H264_FRAME_TYPE_I || frame->frame_type == H264_FRAME_TYPE_P){
            pemft = &emft_slice;
            pfm = &fm_slice;
            nal = &frame->nal;

            buf_offset += sizeof(struct isil_common_stream_header);
            buf_size -= sizeof(struct isil_common_stream_header);
            picsh->stream_type = ISIL_H264_STERAM;
            picsh->stream_len = sizeof(struct isil_common_stream_header);

            buf_offset += sizeof(isil_transparent_msg_header_t);
            buf_size -= sizeof(isil_transparent_msg_header_t);
            pmh->isil_transparent_msg_number = 2;///////
            picsh->stream_len += sizeof(isil_transparent_msg_header_t);
            pmh->isil_transparent_msg_total_len = sizeof(isil_transparent_msg_header_t);

            buf_offset += sizeof(isil_h264_encode_msg_frame_type_t);
            buf_size -= sizeof(isil_h264_encode_msg_frame_type_t);
            pemft->msg_type = ISIL_H264_ENCODE_FRAME_TYPE;
            pemft->msg_len = sizeof(isil_h264_encode_msg_frame_type_t);
            pemft->frame_type = frame->frame_type + 2;
            pemft->i_mb_x = frame->i_mb_x;
            pemft->i_mb_y = frame->i_mb_y;
            picsh->stream_len += sizeof(isil_h264_encode_msg_frame_type_t);
            pmh->isil_transparent_msg_total_len += sizeof(isil_h264_encode_msg_frame_type_t);

            buf_offset += sizeof(isil_frame_msg_t);
            buf_size -= sizeof(isil_frame_msg_t);
            pfm->msg_type = ISIL_H264_ENCODE_FRAME_MSG;
            pfm->msg_len  = sizeof(isil_frame_msg_t);
            pfm->frame_serial = frame->frame_number;
            pfm->frame_timestamp = frame->timestamp;
            pfm->frame_checksam = 0;
            picsh->stream_len += sizeof(isil_frame_msg_t);
            pmh->isil_transparent_msg_total_len += sizeof(isil_frame_msg_t);

            //ISIL_DBG(ISIL_DBG_ERR,"frame_number[%d]\n",frame->frame_number);

            expect_len = tc_align_up(frame->frame_len, 4);
            if(expect_len <= buf_size){
                video_packet_queue = &frame->video_packet_queue;
                if(video_packet_queue->op && video_packet_queue->op->get_curr_queue_entry_number){
                    while(video_packet_queue->op->get_curr_queue_entry_number(video_packet_queue)){
                        video_packet_queue->op->get_curr_consumer_from_queue(video_packet_queue);

                        if(encode_chan->if_slience_discard){
                            ISIL_DBG(ISIL_DBG_INFO, "slicent discard! frame->number:[%d]\n", frame->frame_number);
                            if(video_packet_queue->curr_consumer->op && video_packet_queue->curr_consumer->op->slicent_discard){
                                packet_size = video_packet_queue->curr_consumer->op->slicent_discard(video_packet_queue->curr_consumer, &data[buf_offset], buf_size, &packet_offset, nal);
                            }else{
                                ISIL_DBG(ISIL_DBG_INFO, "met NULL pointer in frame->number:[%d]\n", frame->frame_number);
                                goto exception;
                            }
                        }else{
                            if(video_packet_queue->curr_consumer->op && video_packet_queue->curr_consumer->op->submit){
                                crc_calc_base = &data[buf_offset];
                                packet_size = video_packet_queue->curr_consumer->op->submit(video_packet_queue->curr_consumer, &data[buf_offset], buf_size, &packet_offset, nal);
				crc_buf = vmalloc(packet_size);
				copy_from_user(crc_buf, crc_calc_base, packet_size);
                                pfm->frame_checksam = compute_accumulate_crc(pfm->frame_checksam, crc_buf, packet_size);
				vfree(crc_buf);
                            }else{
                                ISIL_DBG(ISIL_DBG_INFO, "met NULL pointer in frame->number:[%d]\n", frame->frame_number);
                                goto exception;
                            }
                        }
                        video_packet_queue->op->release_curr_consumer(video_packet_queue, video_chan_buf_pool);
                        buf_size -= packet_size;
                        buf_offset += packet_size;
                        picsh->stream_len += packet_size;
                        pmh->isil_transparent_msg_total_len += packet_size;
                        pfm->msg_len += packet_size;
                    }
                    if(!encode_chan->if_slience_discard){
                        while(buf_offset & 3){
                            __put_user(0, &data[buf_offset]);
                            buf_size--;
                            buf_offset++;
                            picsh->stream_len++;
                            pmh->isil_transparent_msg_total_len++;
                            pfm->msg_len++;
                        }
                    }else{
                        //if the slience discard set, we may discard this frame, return 0 to user
                        goto exception;
                    }
                }else{
                    ISIL_DBG(ISIL_DBG_INFO, "met NULL pointer \n");
                    goto exception;
                }
            }else{
                ISIL_DBG(ISIL_DBG_INFO, "buffer size is smaller than one frame!!!\n");
                encode_chan->if_slience_discard = 0;
                goto exception;
            }

            picsh->stream_len |= ISIL_COMMON_STREAM_HEADER_HAVE_PADDING_MSG;
            picsh->stream_timestamp = frame->timestamp;

            /*finally we copy the header info to the head of the user buffer*/
            pc = (char *)data;
            if(copy_to_user(pc, &icsh, sizeof(struct isil_common_stream_header)) != 0){
                ISIL_DBG(ISIL_DBG_INFO, "struct isil_common_stream_header copy_to_user failed\n");
                goto exception;
            }
            pc += sizeof(struct isil_common_stream_header);
            if(copy_to_user(pc, &mh, sizeof(isil_transparent_msg_header_t)) != 0){
                ISIL_DBG(ISIL_DBG_INFO, "isil_transparent_msg_header_t copy_to_user failed\n");
                goto exception;
            }
            pc += sizeof(isil_transparent_msg_header_t);
            if(copy_to_user(pc, &emft_slice, sizeof(isil_h264_encode_msg_frame_type_t)) != 0){
                ISIL_DBG(ISIL_DBG_INFO, "isil_h264_encode_msg_frame_type_t copy_to_user failed\n");
                goto exception;
            }
            pc += sizeof(isil_h264_encode_msg_frame_type_t);
            if(copy_to_user(pc, &fm_slice, sizeof(isil_frame_msg_t)) != 0){
                ISIL_DBG(ISIL_DBG_INFO, "isil_frame_msg_t copy_to_user failed\n");
                goto exception;
            }
        }else if(frame->frame_type == H264_FRAME_TYPE_IDR){
            if(encode_chan->if_slience_discard)
                encode_chan->if_slience_discard  = 0;

            pemft = &emft_sps;
            pfm = &fm_sps;
            nal = &frame->nal;
            buf_offset += sizeof(struct isil_common_stream_header);
            buf_size -= sizeof(struct isil_common_stream_header);
            picsh->stream_type = ISIL_H264_STERAM;
            picsh->stream_len = sizeof(struct isil_common_stream_header);

            buf_offset += sizeof(isil_transparent_msg_header_t);
            buf_size -= sizeof(isil_transparent_msg_header_t);
            pmh->isil_transparent_msg_number = 6;///////
            picsh->stream_len += sizeof(isil_transparent_msg_header_t);
            pmh->isil_transparent_msg_total_len = sizeof(isil_transparent_msg_header_t);

            /*sps frame*/
            pc = (char *)&data[buf_offset];
            buf_offset += sizeof(isil_h264_encode_msg_frame_type_t);
            buf_size -= sizeof(isil_h264_encode_msg_frame_type_t);
            pemft->msg_type = ISIL_H264_ENCODE_FRAME_TYPE;
            pemft->msg_len = sizeof(isil_h264_encode_msg_frame_type_t);
            pemft->frame_type = ISIL_SPS_FRAME;
            pemft->i_mb_x = frame->i_mb_x;
            pemft->i_mb_y = frame->i_mb_y;
            picsh->stream_len += sizeof(isil_h264_encode_msg_frame_type_t);
            pmh->isil_transparent_msg_total_len += sizeof(isil_h264_encode_msg_frame_type_t);

            buf_offset += sizeof(isil_frame_msg_t);
            buf_size -= sizeof(isil_frame_msg_t);
            pfm->msg_type = ISIL_H264_ENCODE_FRAME_MSG;
            pfm->msg_len  = sizeof(isil_frame_msg_t);
            pfm->frame_serial = frame->frame_number;
            pfm->frame_timestamp = frame->timestamp;
            pfm->frame_checksam = 0;
            picsh->stream_len += sizeof(isil_frame_msg_t);
            pmh->isil_transparent_msg_total_len += sizeof(isil_frame_msg_t);

            video_packet_queue = &frame->video_packet_queue;
            if(!video_packet_queue->op || !video_packet_queue->op->get_curr_consumer_from_queue){
                ISIL_DBG(ISIL_DBG_INFO, "met NULL pointer\n");
                goto exception;
            }
            video_packet_queue->op->get_curr_consumer_from_queue(video_packet_queue);
            if(NULL == video_packet_queue->curr_consumer){
                ISIL_DBG(ISIL_DBG_INFO, "BUG! video frame has no packet!");
                goto exception;
            }
            pfm->frame_checksam = compute_crc(video_packet_queue->curr_consumer->data, nal->sps_paysize);
            if(copy_to_user(&data[buf_offset],video_packet_queue->curr_consumer->data, nal->sps_paysize)){
                ISIL_DBG(ISIL_DBG_INFO, "sps copy_to_user failed\n");
                video_packet_queue->op->release_curr_consumer(video_packet_queue, video_chan_buf_pool);
                goto exception;
            }
            pfm->msg_len += nal->sps_paysize;
            if(copy_to_user(pc, &emft_sps, sizeof(isil_h264_encode_msg_frame_type_t)) != 0){
                ISIL_DBG(ISIL_DBG_INFO, "isil_h264_encode_msg_frame_type_t copy_to_user failed\n");
                video_packet_queue->op->release_curr_consumer(video_packet_queue, video_chan_buf_pool);
                goto exception;
            }
            pc += sizeof(isil_h264_encode_msg_frame_type_t);
            if(copy_to_user(pc, &fm_sps, sizeof(isil_frame_msg_t)) != 0){
                ISIL_DBG(ISIL_DBG_INFO, "isil_frame_msg_t copy_to_user failed\n");
                video_packet_queue->op->release_curr_consumer(video_packet_queue, video_chan_buf_pool);
                goto exception;
            }      


            /*pps frame*/
            buf_offset += nal->sps_paysize;
            buf_size -= nal->sps_paysize;
            picsh->stream_len += nal->sps_paysize;
            pmh->isil_transparent_msg_total_len += nal->sps_paysize;
            pemft = &emft_pps;
            pfm = &fm_pps;
            pc = (char *)&data[buf_offset];

            buf_offset += sizeof(isil_h264_encode_msg_frame_type_t);
            buf_size -= sizeof(isil_h264_encode_msg_frame_type_t);
            pemft->msg_type = ISIL_H264_ENCODE_FRAME_TYPE;
            pemft->msg_len = sizeof(isil_h264_encode_msg_frame_type_t);
            pemft->frame_type = ISIL_PPS_FRAME;
            pemft->i_mb_x = frame->i_mb_x;
            pemft->i_mb_y = frame->i_mb_y;
            picsh->stream_len += sizeof(isil_h264_encode_msg_frame_type_t);
            pmh->isil_transparent_msg_total_len += sizeof(isil_h264_encode_msg_frame_type_t);

            buf_offset += sizeof(isil_frame_msg_t);
            buf_size -= sizeof(isil_frame_msg_t);
            pfm->msg_type = ISIL_H264_ENCODE_FRAME_MSG;
            pfm->msg_len  = sizeof(isil_frame_msg_t);
            pfm->frame_serial = frame->frame_number;
            pfm->frame_timestamp = frame->timestamp;
            pfm->frame_checksam = 0;
            picsh->stream_len += sizeof(isil_frame_msg_t);
            pmh->isil_transparent_msg_total_len += sizeof(isil_frame_msg_t);

            pfm->frame_checksam = compute_crc(&video_packet_queue->curr_consumer->data[nal->sps_paysize], nal->pps_paysize);
            if(copy_to_user(&data[buf_offset],&video_packet_queue->curr_consumer->data[nal->sps_paysize], nal->pps_paysize)){
                ISIL_DBG(ISIL_DBG_INFO, "pps copy_to_user failed\n");
                video_packet_queue->op->release_curr_consumer(video_packet_queue, video_chan_buf_pool);
                goto exception;
            }
            pfm->msg_len += nal->pps_paysize;
            if(copy_to_user(pc, &emft_pps, sizeof(isil_h264_encode_msg_frame_type_t)) != 0){
                ISIL_DBG(ISIL_DBG_INFO, "isil_h264_encode_msg_frame_type_t copy_to_user failed\n");
                video_packet_queue->op->release_curr_consumer(video_packet_queue, video_chan_buf_pool);
                goto exception;
            }
            pc += sizeof(isil_h264_encode_msg_frame_type_t);
            if(copy_to_user(pc, &fm_pps, sizeof(isil_frame_msg_t)) != 0){
                ISIL_DBG(ISIL_DBG_INFO, "isil_frame_msg_t copy_to_user failed\n");
                video_packet_queue->op->release_curr_consumer(video_packet_queue, video_chan_buf_pool);
                goto exception;
            }

            /*IDR slice frame*/
            buf_offset += nal->pps_paysize;
            buf_size -= nal->pps_paysize;
            picsh->stream_len += nal->pps_paysize;
            pmh->isil_transparent_msg_total_len += nal->pps_paysize;
            pemft = &emft_slice;
            pfm = &fm_slice;
            pc = (char *)&data[buf_offset];

            buf_offset += sizeof(isil_h264_encode_msg_frame_type_t);
            buf_size -= sizeof(isil_h264_encode_msg_frame_type_t);
            pemft->msg_type = ISIL_H264_ENCODE_FRAME_TYPE;
            pemft->msg_len = sizeof(isil_h264_encode_msg_frame_type_t);
            pemft->frame_type = ISIL_IDR_FRAME;
            pemft->i_mb_x = frame->i_mb_x;
            pemft->i_mb_y = frame->i_mb_y;
            picsh->stream_len += sizeof(isil_h264_encode_msg_frame_type_t);
            pmh->isil_transparent_msg_total_len += sizeof(isil_h264_encode_msg_frame_type_t);

            buf_offset += sizeof(isil_frame_msg_t);
            buf_size -= sizeof(isil_frame_msg_t);
            pfm->msg_type = ISIL_H264_ENCODE_FRAME_MSG;
            pfm->msg_len  = sizeof(isil_frame_msg_t);
            pfm->frame_serial = frame->frame_number;
            pfm->frame_timestamp = frame->timestamp;
            pfm->frame_checksam = 0;
            picsh->stream_len += sizeof(isil_frame_msg_t);
            pmh->isil_transparent_msg_total_len += sizeof(isil_frame_msg_t);

            expect_len = tc_align_up(frame->frame_len, 4);
            if(expect_len > buf_size){
                ISIL_DBG(ISIL_DBG_INFO, "no enough buffer lefted in function!\n");
                encode_chan->if_slience_discard  = 0;
            }

            /*copy the rest of the first packet, then release it*/
            video_packet_queue->curr_consumer->consumer_len = (nal->sps_paysize + nal->pps_paysize);
            crc_calc_base = &data[buf_offset];
            packet_size = video_packet_queue->curr_consumer->op->submit(video_packet_queue->curr_consumer, &data[buf_offset], buf_size, &packet_offset, nal);
	    crc_buf = vmalloc(packet_size);
	    copy_from_user(crc_buf, pc, packet_size);
	    pfm->frame_checksam = compute_crc(crc_buf, packet_size);
	    vfree(crc_buf);
            video_packet_queue->op->release_curr_consumer(video_packet_queue, video_chan_buf_pool);
            buf_size -= packet_size;
            buf_offset += packet_size;
            picsh->stream_len += packet_size;
            pmh->isil_transparent_msg_total_len += packet_size;
            pfm->msg_len += packet_size;

            /*copey the rest packets*/
            if(!video_packet_queue->op || !video_packet_queue->op->get_curr_queue_entry_number){
                ISIL_DBG(ISIL_DBG_INFO, "BUG: previous is ok, but now met NULL pointer!\n");
                goto exception;
            }
            while(video_packet_queue->op->get_curr_queue_entry_number(video_packet_queue)){
                video_packet_queue->op->get_curr_consumer_from_queue(video_packet_queue);
                if(encode_chan->if_slience_discard){
                    if(!video_packet_queue->curr_consumer->op || !video_packet_queue->curr_consumer->op->slicent_discard){
                        ISIL_DBG(ISIL_DBG_INFO, "BUG: previous is ok, but now met NULL pointer!\n");
                        goto exception;
                    }
                    packet_size = video_packet_queue->curr_consumer->op->slicent_discard(video_packet_queue->curr_consumer, &data[buf_offset], buf_size, &packet_offset, nal);
                }else{
                    if(!video_packet_queue->curr_consumer->op || !video_packet_queue->curr_consumer->op->submit){
                        ISIL_DBG(ISIL_DBG_INFO, "BUG: previous is ok, but now met NULL pointer!\n");
                        goto exception;
                    }
                    crc_calc_base = &data[buf_offset];
                    packet_size = video_packet_queue->curr_consumer->op->submit(video_packet_queue->curr_consumer, &data[buf_offset], buf_size, &packet_offset, nal);
		    crc_buf = vmalloc(packet_size);
		    copy_from_user(crc_buf, crc_calc_base, packet_size);
                    pfm->frame_checksam = compute_accumulate_crc(pfm->frame_checksam, crc_buf, packet_size);
		    vfree(crc_buf);
                }
                if(!video_packet_queue->op || !video_packet_queue->op->release_curr_consumer){
                    ISIL_DBG(ISIL_DBG_INFO, "BUG: previous is ok, but now met NULL pointer!\n");
                    goto exception;
                }
                video_packet_queue->op->release_curr_consumer(video_packet_queue, video_chan_buf_pool);
                buf_size -= packet_size;
                buf_offset += packet_size;
                picsh->stream_len += packet_size;
                pmh->isil_transparent_msg_total_len += packet_size;
                pfm->msg_len += packet_size;
            }
            if(!encode_chan->if_slience_discard){
                while(buf_offset & 3){
                    __put_user(0, &data[buf_offset]);
                    buf_size--;
                    buf_offset++;
                    picsh->stream_len++;
                    pmh->isil_transparent_msg_total_len++;
                    pfm->msg_len++;
                }
            }else{
                goto exception;
            }

            picsh->stream_len |= ISIL_COMMON_STREAM_HEADER_HAVE_PADDING_MSG;
            if(copy_to_user(pc, &emft_slice, sizeof(isil_h264_encode_msg_frame_type_t)) != 0){
                ISIL_DBG(ISIL_DBG_INFO, " isil_h264_encode_msg_frame_type_t copy_to_user failed\n");
                goto exception;
            }
            pc += sizeof(isil_h264_encode_msg_frame_type_t);
            if(copy_to_user(pc, &fm_slice, sizeof(isil_frame_msg_t)) != 0){
                ISIL_DBG(ISIL_DBG_INFO, " isil_frame_msg_t copy_to_user failed\n");
                goto exception;
            }

            pc = (char *)data;
            if(copy_to_user(pc, &icsh, sizeof(struct isil_common_stream_header)) != 0){
                ISIL_DBG(ISIL_DBG_INFO, " struct isil_common_stream_header copy_to_user failed\n");
                goto exception;
            }
            pc += sizeof(struct isil_common_stream_header);
            if(copy_to_user(pc, &mh, sizeof(isil_transparent_msg_header_t)) != 0){
                ISIL_DBG(ISIL_DBG_INFO, " isil_transparent_msg_header_t copy_to_user failed\n");
                goto exception;
            }
        }
    }else{
        ISIL_DBG(ISIL_DBG_INFO, "frame is NULL\n");
    }

    *ppos += buf_offset;
    return (size_t)buf_offset;
exception:
    *ppos = 0;
    return 0;
}
#endif



/****************************************************************************************************************
* Function Name :   isil_video_frame_tcb_copy
* Description   :   copy video frame
* Input         :   param1 is src video frame
*                   param2 is dest video frame
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static int isil_video_frame_tcb_copy(isil_video_frame_tcb_t *src, isil_video_frame_tcb_t *dst)
{
    if(src && dst) {
        u32 entry_number;
        int i;
        h264_nal_t *src_nal, *dst_nal;
        isil_video_chan_buf_pool_t *video_chan_buf_pool;
        isil_video_packet_tcb_queue_t	*src_packet_queue, *dst_packet_queue;

        src_packet_queue = &src->video_packet_queue;
        dst_packet_queue = &dst->video_packet_queue;

        video_chan_buf_pool = dst->video_chan_buf_pool;
        entry_number = src_packet_queue->op->get_curr_queue_entry_number(src_packet_queue);
        for(i = 0; i < entry_number; i++) {
            isil_video_packet_tcb_t *src_packet, *dst_packet;

            video_chan_buf_pool->op->try_get_video_packet_tcb(video_chan_buf_pool, &dst_packet);
            src_packet_queue->op->try_get(src_packet_queue, &src_packet);
            if(src_packet && dst_packet) {
                src_packet->op->copy(src_packet, dst_packet);
                dst_packet_queue->op->put(dst_packet_queue, dst_packet);
            }else{
                dst->frame_is_err = 1;
            }
            src_packet_queue->op->put(src_packet_queue, src_packet);
        }

        dst->b_is_display           = src->b_is_display;
        dst->b_is_gop_end           = src->b_is_gop_end;
        dst->checksum               = src->checksum;
        dst->consumer_frame_offset  = src->consumer_frame_offset;
        dst->fps                    = src->fps;
        if(dst->frame_is_err == 0) {
            dst->frame_is_err           = src->frame_is_err;
        }
        dst->frame_len              = src->frame_len;
        dst->frame_number           = src->frame_number;
        dst->frame_type             = src->frame_type;
        dst->i_curr_qp              = src->i_curr_qp;
        dst->i_init_qp              = src->i_init_qp;
        dst->i_log_gop_value        = src->i_log_gop_value;
        dst->i_mb_x                 = src->i_mb_x;
        dst->i_mb_y                 = src->i_mb_y;
        dst->in_ddr_addr            = src->in_ddr_addr;
        dst->slice_type             = src->slice_type;
        dst->in_ddr_addr            = src->in_ddr_addr;
        dst->timestamp              = src->timestamp;
        dst->duration               = src->duration;
        memcpy(&dst->time_pad, &src->time_pad, sizeof(ISIL_FRAME_TIME_PAD));

        src_nal = &src->nal;
        dst_nal = &dst->nal;
        dst_nal->header_align_number     = src_nal->header_align_number;
        dst_nal->i_end_bitalign_number   = src_nal->i_end_bitalign_number;
        dst_nal->i_ref_idc               = src_nal->i_ref_idc;
        dst_nal->i_temp_bitalign_content = src_nal->i_temp_bitalign_content;
        dst_nal->i_temp_bitalign_number  = src_nal->i_temp_bitalign_number;
        dst_nal->i_type                  = src_nal->i_type;
        dst_nal->pps_paysize             = src_nal->pps_paysize;
        dst_nal->slice_paysize           = src_nal->slice_paysize;
        dst_nal->sps_paysize             = src_nal->sps_paysize;
        dst_nal->slice_head_buf          = NULL;
        dst_nal->sps_payload             = NULL;
        dst_nal->pps_payload             = NULL;

        if(dst->frame_is_err) {
            return ISIL_ERR;
        }
        return ISIL_OK;
    }

    return ISIL_ERR;
}
static struct isil_video_frame_tcb_operation  isil_video_frame_tcb_op = {
    .init = isil_video_frame_tcb_init,
    .reset = isil_video_frame_tcb_reset,
    .release = isil_video_frame_tcb_release,
    .reference = isil_video_frame_tcb_reference,
    .submit = isil_video_frame_tcb_submit,
    .copy   = isil_video_frame_tcb_copy,
};

/****************************************************************************************************************
* Function Name :   isil_video_frame_queue_get
* Description   :   get video frame from queue
* Input         :   param1 is video frame queue
* Output        :   video frame
* Return        :   null
****************************************************************************************************************/
static void isil_video_frame_queue_get(isil_video_frame_tcb_queue_t *video_frame_tcb_queue, isil_video_frame_tcb_t **ptr_frame)
{
    if((video_frame_tcb_queue!=NULL) && (ptr_frame!=NULL)){
        tcb_node_queue_t	*queue;
        *ptr_frame = NULL;
        queue = &video_frame_tcb_queue->queue_node;
        if(queue->op != NULL){
            tcb_node_t	*temp_node;
            queue->op->get(queue, &temp_node);
            if(temp_node != NULL){
                *ptr_frame = to_video_frame_tcb(temp_node);
            }
        }
    } else {
        ISIL_DBG(ISIL_DBG_INFO, ": **attention**\n");
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_frame_queue_try_get
* Description   :   try get video frame from queue
* Input         :   param1 is video frame queue
* Output        :   video frame
* Return        :   null
****************************************************************************************************************/
static void isil_video_frame_queue_try_get(isil_video_frame_tcb_queue_t *video_frame_tcb_queue, isil_video_frame_tcb_t **ptr_frame)
{
    if((video_frame_tcb_queue!=NULL) && (ptr_frame!=NULL)){
        tcb_node_queue_t    *queue;
        *ptr_frame = NULL;
        queue = &video_frame_tcb_queue->queue_node;
        if(queue->op != NULL){
            tcb_node_t	*temp_node;
            queue->op->try_get(queue, &temp_node);
            if(temp_node != NULL){
                *ptr_frame = to_video_frame_tcb(temp_node);
            }
        }
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_frame_queue_get_tailer
* Description   :   get video frame from queue tailer
* Input         :   param1 is video frame queue
* Output        :   video frame
* Return        :   null
****************************************************************************************************************/
static void isil_video_frame_queue_get_tailer(isil_video_frame_tcb_queue_t *video_frame_tcb_queue, isil_video_frame_tcb_t **ptr_frame)
{
    if((video_frame_tcb_queue!=NULL) && (ptr_frame!=NULL)){
        tcb_node_queue_t    *queue;
        *ptr_frame = NULL;
        queue = &video_frame_tcb_queue->queue_node;
        if(queue->op != NULL){
            tcb_node_t	*temp_node;
            queue->op->get_tailer(queue, &temp_node);
            if(temp_node != NULL){
                *ptr_frame = to_video_frame_tcb(temp_node);
            }
        }
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_frame_queue_try_get_tailer
* Description   :   try get video frame from queue tailer
* Input         :   param1 is video frame queue
* Output        :   video frame
* Return        :   null
****************************************************************************************************************/
static void isil_video_frame_queue_try_get_tailer(isil_video_frame_tcb_queue_t *video_frame_tcb_queue, isil_video_frame_tcb_t **ptr_frame)
{
    if((video_frame_tcb_queue!=NULL) && (ptr_frame!=NULL)){
        tcb_node_queue_t    *queue;
        *ptr_frame = NULL;
        queue = &video_frame_tcb_queue->queue_node;
        if(queue->op != NULL){
            tcb_node_t	*temp_node;
            queue->op->try_get_tailer(queue, &temp_node);
            if(temp_node != NULL){
                *ptr_frame = to_video_frame_tcb(temp_node);
            }
        }
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_frame_queue_put
* Description   :   put video frame into queue
* Input         :   param1 is video frame queue
*                   param2 is video frame
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void isil_video_frame_queue_put(isil_video_frame_tcb_queue_t *video_frame_tcb_queue, isil_video_frame_tcb_t *frame)
{
    if((video_frame_tcb_queue!=NULL) && (frame!=NULL)){
        tcb_node_queue_t    *queue;
        queue = &video_frame_tcb_queue->queue_node;
        if(queue->op != NULL){
            queue->op->put(queue, &frame->frame_node);
        }
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_frame_queue_put_header
* Description   :   put video frame into queue header
* Input         :   param1 is video frame queue
*                   param2 is video frame
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void isil_video_frame_queue_put_header(isil_video_frame_tcb_queue_t *video_frame_tcb_queue, isil_video_frame_tcb_t *frame)
{
    if((video_frame_tcb_queue!=NULL) && (frame!=NULL)){
        tcb_node_queue_t    *queue;
        queue = &video_frame_tcb_queue->queue_node;
        if(queue->op != NULL){
            queue->op->put_header(queue, &frame->frame_node);
        }
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_frame_queue_get_curr_producer_from_pool
* Description   :   get video frame into pool
* Input         :   param1 is video frame queue
*                   param2 is video buf pool
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void isil_video_frame_queue_get_curr_producer_from_pool(isil_video_frame_tcb_queue_t *video_frame_tcb_queue, isil_video_chan_buf_pool_t *video_chan_buf_pool)
{
    if((video_frame_tcb_queue!=NULL) && (video_chan_buf_pool!=NULL)){
        unsigned long	flags;
        spin_lock_irqsave(&video_frame_tcb_queue->lock, flags);
        if(video_frame_tcb_queue->curr_producer == NULL){
            spin_unlock_irqrestore(&video_frame_tcb_queue->lock, flags);
            if(video_chan_buf_pool->op != NULL){
                video_chan_buf_pool->op->get_video_frame_tcb(video_chan_buf_pool, &video_frame_tcb_queue->curr_producer);
            }
            spin_lock_irqsave(&video_frame_tcb_queue->lock, flags);
        }
        spin_unlock_irqrestore(&video_frame_tcb_queue->lock, flags);
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_frame_queue_try_get_curr_producer_from_pool
* Description   :   try get video frame into pool
* Input         :   param1 is video frame queue
*                   param2 is video buf pool
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void isil_video_frame_queue_try_get_curr_producer_from_pool(isil_video_frame_tcb_queue_t *video_frame_tcb_queue, isil_video_chan_buf_pool_t *video_chan_buf_pool)
{
    if((video_frame_tcb_queue!=NULL) && (video_chan_buf_pool!=NULL)){
        unsigned long	flags;
        spin_lock_irqsave(&video_frame_tcb_queue->lock, flags);
        if(video_frame_tcb_queue->curr_producer == NULL){
            if(video_chan_buf_pool->op != NULL){
                video_chan_buf_pool->op->try_get_video_frame_tcb(video_chan_buf_pool, &video_frame_tcb_queue->curr_producer);
            }
        }
        spin_unlock_irqrestore(&video_frame_tcb_queue->lock, flags);
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_frame_queue_put_curr_producer_into_queue
* Description   :   put video frame into queue
* Input         :   param1 is video frame queue
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void isil_video_frame_queue_put_curr_producer_into_queue(isil_video_frame_tcb_queue_t *video_frame_tcb_queue)
{
    if(video_frame_tcb_queue != NULL){
        unsigned long	flags;
        spin_lock_irqsave(&video_frame_tcb_queue->lock, flags);
        if(video_frame_tcb_queue->curr_producer != NULL){
            if(video_frame_tcb_queue->op != NULL){
                video_frame_tcb_queue->op->put(video_frame_tcb_queue, video_frame_tcb_queue->curr_producer);
            } else {
                isil_video_frame_queue_put(video_frame_tcb_queue, video_frame_tcb_queue->curr_producer);
            }
            video_frame_tcb_queue->curr_producer = NULL;
        }
        spin_unlock_irqrestore(&video_frame_tcb_queue->lock, flags);
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_frame_queue_release_curr_producer
* Description   :   release curr producer
* Input         :   param1 is video frame queue
*                   param2 is video buf pool
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void isil_video_frame_queue_release_curr_producer(isil_video_frame_tcb_queue_t *video_frame_tcb_queue, isil_video_chan_buf_pool_t *video_chan_buf_pool)
{
    if((video_frame_tcb_queue!=NULL) && (video_chan_buf_pool!=NULL)){
        unsigned long	flags;
        spin_lock_irqsave(&video_frame_tcb_queue->lock, flags);
        if(video_frame_tcb_queue->curr_producer != NULL){
            if(video_frame_tcb_queue->curr_producer->op != NULL){
                video_frame_tcb_queue->curr_producer->op->release(&video_frame_tcb_queue->curr_producer, video_chan_buf_pool);
            } else {
                isil_video_frame_tcb_release(&video_frame_tcb_queue->curr_producer, video_chan_buf_pool);
            }
        }
        spin_unlock_irqrestore(&video_frame_tcb_queue->lock, flags);
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_frame_queue_get_curr_consumer_from_queue
* Description   :   get curr consumer from queue
* Input         :   param1 is video frame queue
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void isil_video_frame_queue_get_curr_consumer_from_queue(isil_video_frame_tcb_queue_t *video_frame_tcb_queue)
{
    if(video_frame_tcb_queue != NULL){
        unsigned long	flags;

        spin_lock_irqsave(&video_frame_tcb_queue->lock, flags);
        if(video_frame_tcb_queue->curr_consumer == NULL){
            spin_unlock_irqrestore(&video_frame_tcb_queue->lock, flags);
            if(video_frame_tcb_queue->op != NULL){
                video_frame_tcb_queue->op->get(video_frame_tcb_queue, &video_frame_tcb_queue->curr_consumer);
            } else {
                isil_video_frame_queue_get(video_frame_tcb_queue, &video_frame_tcb_queue->curr_consumer);
            }
            spin_lock_irqsave(&video_frame_tcb_queue->lock, flags);
        }
        spin_unlock_irqrestore(&video_frame_tcb_queue->lock, flags);
        //printk("get-->%d, %d, %x\n", video_frame_tcb_queue->op->get_curr_queue_entry_number(video_frame_tcb_queue), video_frame_tcb_queue->curr_consumer->timestamp, video_frame_tcb_queue->curr_consumer);
    } else {
        ISIL_DBG(ISIL_DBG_INFO, ": **attention**\n");
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_frame_queue_try_get_curr_consumer_from_queue
* Description   :   try get curr consumer from queue
* Input         :   param1 is video frame queue
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void isil_video_frame_queue_try_get_curr_consumer_from_queue(isil_video_frame_tcb_queue_t *video_frame_tcb_queue)
{
    if(video_frame_tcb_queue != NULL){
        unsigned long	flags;
        spin_lock_irqsave(&video_frame_tcb_queue->lock, flags);
        if(video_frame_tcb_queue->curr_consumer == NULL){
            if(video_frame_tcb_queue->op != NULL){
                video_frame_tcb_queue->op->try_get(video_frame_tcb_queue, &video_frame_tcb_queue->curr_consumer);
            } else {
                isil_video_frame_queue_try_get(video_frame_tcb_queue, &video_frame_tcb_queue->curr_consumer);
            }
        }
        spin_unlock_irqrestore(&video_frame_tcb_queue->lock, flags);
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_frame_queue_release_curr_consumer
* Description   :   release curr consumer
* Input         :   param1 is video frame queue
*                   param2 is video buf pool
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void isil_video_frame_queue_release_curr_consumer(isil_video_frame_tcb_queue_t *video_frame_tcb_queue, isil_video_chan_buf_pool_t *video_chan_buf_pool)
{
    if((video_frame_tcb_queue!=NULL) && (video_chan_buf_pool!=NULL)){
        unsigned long	flags;
        spin_lock_irqsave(&video_frame_tcb_queue->lock, flags);
        if(video_frame_tcb_queue->curr_consumer != NULL){
            if(video_frame_tcb_queue->curr_consumer->op != NULL){
                video_frame_tcb_queue->curr_consumer->op->release(&video_frame_tcb_queue->curr_consumer, video_chan_buf_pool);
            } else {
                isil_video_frame_tcb_release(&video_frame_tcb_queue->curr_consumer, video_chan_buf_pool);
            }
        }
        spin_unlock_irqrestore(&video_frame_tcb_queue->lock, flags);
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_frame_queue_put_curr_consumer_into_queue_header
* Description   :   put curr consumer into queue
* Input         :   param1 is video frame queue
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void isil_video_frame_queue_put_curr_consumer_into_queue_header(isil_video_frame_tcb_queue_t *video_frame_tcb_queue)
{
    if(video_frame_tcb_queue != NULL){
        unsigned long flags;
        spin_lock_irqsave(&video_frame_tcb_queue->lock, flags);
        if(video_frame_tcb_queue->curr_consumer != NULL){
            if(video_frame_tcb_queue->op != NULL){
                video_frame_tcb_queue->op->put_header(video_frame_tcb_queue, video_frame_tcb_queue->curr_consumer);
            } else {
                isil_video_frame_queue_put_header(video_frame_tcb_queue, video_frame_tcb_queue->curr_consumer);
            }
            video_frame_tcb_queue->curr_consumer = NULL;
        }
        spin_unlock_irqrestore(&video_frame_tcb_queue->lock, flags);
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_frame_queue_get_curr_queue_entry_number
* Description   :   get entry number of queue
* Input         :   param1 is video frame queue
* Output        :   null
* Return        :   real number
****************************************************************************************************************/
static int  isil_video_frame_queue_get_curr_queue_entry_number(isil_video_frame_tcb_queue_t *video_frame_tcb_queue)
{
    int	entry_number = 0;
    if(video_frame_tcb_queue!=NULL){
        tcb_node_queue_t	*queue;
        queue = &video_frame_tcb_queue->queue_node;
        if(queue->op != NULL){
            entry_number = queue->op->get_queue_curr_entry_number(queue);
        }
    }
    return entry_number;
}

/****************************************************************************************************************
* Function Name :   isil_video_frame_queue_release
* Description   :   release queue
* Input         :   param1 is video frame queue
*                   param2 is video buf pool
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void isil_video_frame_queue_release(isil_video_frame_tcb_queue_t *video_frame_tcb_queue, isil_video_chan_buf_pool_t *video_chan_buf_pool)
{
    if((video_frame_tcb_queue!=NULL) && (video_chan_buf_pool!=NULL)){
        remove_isil_video_frame_tcb_queue(video_frame_tcb_queue, video_chan_buf_pool);
    }
}

/****************************************************************************************************************
* Function Name :   isil_video_frame_queue_init
* Description   :   init video frame queue
* Input         :   param1 is video frame queue
* Output        :   null
* Return        :   null
****************************************************************************************************************/
static void isil_video_frame_queue_init(isil_video_frame_tcb_queue_t *video_frame_tcb_queue)
{
    if(video_frame_tcb_queue != NULL){
        tcb_node_queue_t    *queue;
        queue = &video_frame_tcb_queue->queue_node;
        queue->op = &tcb_node_queue_op;
        queue->op->init(queue);
        video_frame_tcb_queue->curr_consumer = NULL;
        video_frame_tcb_queue->curr_producer = NULL;
        video_frame_tcb_queue->start_timestamp = 0;
        video_frame_tcb_queue->total_duration = 0;
        spin_lock_init(&video_frame_tcb_queue->lock);
    }
}

static struct isil_video_frame_queue_operation	isil_video_frame_queue_op = {
    .get = isil_video_frame_queue_get,
    .try_get = isil_video_frame_queue_try_get,
    .get_tailer = isil_video_frame_queue_get_tailer,
    .try_get_tailer = isil_video_frame_queue_try_get_tailer,
    .put = isil_video_frame_queue_put,
    .put_header = isil_video_frame_queue_put_header,

    .get_curr_producer_from_pool = isil_video_frame_queue_get_curr_producer_from_pool,
    .try_get_curr_producer_from_pool = isil_video_frame_queue_try_get_curr_producer_from_pool,
    .put_curr_producer_into_queue = isil_video_frame_queue_put_curr_producer_into_queue,
    .release_curr_producer = isil_video_frame_queue_release_curr_producer,
    .get_curr_consumer_from_queue = isil_video_frame_queue_get_curr_consumer_from_queue,
    .try_get_curr_consumer_from_queue = isil_video_frame_queue_try_get_curr_consumer_from_queue,
    .release_curr_consumer = isil_video_frame_queue_release_curr_consumer,
    .put_curr_consumer_into_queue_header = isil_video_frame_queue_put_curr_consumer_into_queue_header,

    .get_curr_queue_entry_number = isil_video_frame_queue_get_curr_queue_entry_number,
    .release = isil_video_frame_queue_release,
    .init = isil_video_frame_queue_init,
};

/****************************************************************************************************************
* Function Name :   init_isil_video_frame_tcb_queue
* Description   :   init video frame queue
* Input         :   param1 is video frame queue
* Output        :   null
* Return        :   null
****************************************************************************************************************/
void    init_isil_video_frame_tcb_queue(isil_video_frame_tcb_queue_t *video_frame_tcb_queue)
{
    if(video_frame_tcb_queue != NULL){
        video_frame_tcb_queue->op = &isil_video_frame_queue_op;
        video_frame_tcb_queue->op->init(video_frame_tcb_queue);
    }
}

/****************************************************************************************************************
* Function Name :   remove_isil_video_frame_tcb_queue
* Description   :   remove video frame queue
* Input         :   param1 is video frame queue
*                   param2 is video buf pool
* Output        :   null
* Return        :   null
****************************************************************************************************************/
void    remove_isil_video_frame_tcb_queue(isil_video_frame_tcb_queue_t *video_frame_tcb_queue, isil_video_chan_buf_pool_t *video_chan_buf_pool)
{
    if((video_frame_tcb_queue!=NULL) && (video_chan_buf_pool!=NULL)){
        if(video_frame_tcb_queue->op != NULL){
            video_frame_tcb_queue->op->release_curr_producer(video_frame_tcb_queue, video_chan_buf_pool);
            video_frame_tcb_queue->op->release_curr_consumer(video_frame_tcb_queue, video_chan_buf_pool);
            while(video_frame_tcb_queue->op->get_curr_queue_entry_number(video_frame_tcb_queue)){
                video_frame_tcb_queue->op->get_curr_consumer_from_queue(video_frame_tcb_queue);
                if(video_frame_tcb_queue->curr_consumer == NULL){
                    break;
                }
                video_frame_tcb_queue->op->release_curr_consumer(video_frame_tcb_queue, video_chan_buf_pool);
            }
        } else {
            isil_video_frame_queue_release_curr_producer(video_frame_tcb_queue, video_chan_buf_pool);
            isil_video_frame_queue_release_curr_consumer(video_frame_tcb_queue, video_chan_buf_pool);
            while(isil_video_frame_queue_get_curr_queue_entry_number(video_frame_tcb_queue)){
                isil_video_frame_queue_get_curr_consumer_from_queue(video_frame_tcb_queue);
                if(video_frame_tcb_queue->curr_consumer == NULL){
                    break;
                }
                isil_video_frame_queue_release_curr_consumer(video_frame_tcb_queue, video_chan_buf_pool);
            }
        }
    }
}

static int	isil_video_chan_buf_pool_create(isil_video_chan_buf_pool_t *video_chan_buf_pool, unsigned long buf_len)
{
    int	ret = ISIL_ERR;

    if(video_chan_buf_pool != NULL){
        tcb_node_pool_t	*node_pool;
        isil_video_packet_tcb_t   *ptr_video_packet_tcb;
        isil_video_frame_tcb_t    *ptr_video_frame_tcb;
        int i, entry_number;

        video_chan_buf_pool->cache_order = get_order(buf_len);
        entry_number = 1;
        for(i=0; i<video_chan_buf_pool->cache_order; i++){
            entry_number <<= 1;
        }
        video_chan_buf_pool->video_packet_buffer_size = PAGE_SIZE;

        node_pool = &video_chan_buf_pool->video_packet_pool_tcb;
        node_pool->op = &tcb_node_pool_op;
        video_chan_buf_pool->video_packet_entry_pool = (isil_video_packet_tcb_t*)kzalloc(sizeof(isil_video_packet_tcb_t)*entry_number, GFP_KERNEL);
        if(video_chan_buf_pool->video_packet_entry_pool == NULL){
            video_chan_buf_pool->cache_order = 0;
            ISIL_DBG(ISIL_DBG_INFO, ": can't alloc video packet tcb\n");
            return ret;
        }
        node_pool->op->init(node_pool, entry_number);
        for(i=0; i<entry_number; i++){
            ptr_video_packet_tcb = &video_chan_buf_pool->video_packet_entry_pool[i];
            ptr_video_packet_tcb->op = &isil_video_packet_tcb_op;
            ret = ptr_video_packet_tcb->op->init(ptr_video_packet_tcb, video_chan_buf_pool, i);
            if(ret < 0){
                node_pool->op->init(node_pool, 0);
                ISIL_DBG(ISIL_DBG_ERR, "init video packet page failed %d\n", ret);
                ret = ISIL_ERR;
                break;
            }
            ptr_video_packet_tcb->op->release(&ptr_video_packet_tcb, video_chan_buf_pool);
            video_chan_buf_pool->video_packet_entry_number++;
        }
        if(ret == ISIL_ERR){
            for(i=0; i<video_chan_buf_pool->video_packet_entry_number; i++){
                ptr_video_packet_tcb = &video_chan_buf_pool->video_packet_entry_pool[i];
                if(ptr_video_packet_tcb->data){
                    free_page((unsigned long) ptr_video_packet_tcb->data);
                    ptr_video_packet_tcb->data = NULL;
                }
            }
            kfree(video_chan_buf_pool->video_packet_entry_pool);
            video_chan_buf_pool->video_packet_entry_pool = NULL;
            video_chan_buf_pool->video_packet_entry_number = 0;
            video_chan_buf_pool->cache_order = 0;
            ISIL_DBG(ISIL_DBG_INFO, ": can't alloc video packet page\n");
            return ret;
        }

        node_pool = &video_chan_buf_pool->video_frame_pool_tcb;
        node_pool->op = &tcb_node_pool_op;
        video_chan_buf_pool->video_frame_entry_pool = (isil_video_frame_tcb_t*)kzalloc(sizeof(isil_video_frame_tcb_t)*entry_number, GFP_KERNEL);
        if(video_chan_buf_pool->video_frame_entry_pool == NULL){
            node_pool = &video_chan_buf_pool->video_packet_pool_tcb;
            node_pool->op->release(node_pool);
            for(i=0; i<video_chan_buf_pool->video_packet_entry_number; i++){
                ptr_video_packet_tcb = &video_chan_buf_pool->video_packet_entry_pool[i];
                if(ptr_video_packet_tcb->data){
                    free_page((unsigned long) ptr_video_packet_tcb->data);
                    ptr_video_packet_tcb->data = NULL;
                }
            }
            kfree(video_chan_buf_pool->video_packet_entry_pool);
            video_chan_buf_pool->video_packet_entry_pool = NULL;
            video_chan_buf_pool->video_packet_entry_number = 0;
            video_chan_buf_pool->cache_order = 0;
            node_pool->op->init(node_pool, 0);
            ISIL_DBG(ISIL_DBG_INFO, ": can't alloc video frame tcb\n");
            ret = ISIL_ERR;
            return ret;
        }
        node_pool->op->init(node_pool, entry_number);
        for(i=0; i<entry_number; i++){
            ptr_video_frame_tcb = &video_chan_buf_pool->video_frame_entry_pool[i];
            ptr_video_frame_tcb->op = &isil_video_frame_tcb_op;
            ptr_video_frame_tcb->video_chan_buf_pool = video_chan_buf_pool;
            ptr_video_frame_tcb->op->init(ptr_video_frame_tcb);
            ptr_video_frame_tcb->op->release(&ptr_video_frame_tcb, video_chan_buf_pool);
            video_chan_buf_pool->video_frame_entry_number++;
        }

        ret = ISIL_OK;
    }
    return ret;
}

static void isil_video_chan_buf_pool_release(isil_video_chan_buf_pool_t *video_chan_buf_pool)
{
    if(video_chan_buf_pool != NULL){
        tcb_node_pool_t	*node_pool;

        node_pool = &video_chan_buf_pool->video_frame_pool_tcb;
        if(node_pool->op != NULL){
            node_pool->op->release(node_pool);
        }
        if(video_chan_buf_pool->video_frame_entry_pool != NULL){
            kfree(video_chan_buf_pool->video_frame_entry_pool);
            video_chan_buf_pool->video_frame_entry_pool = NULL;
        }
        video_chan_buf_pool->video_frame_entry_number = 0;

        node_pool = &video_chan_buf_pool->video_packet_pool_tcb;
        if(node_pool->op != NULL){
            node_pool->op->release(node_pool);
        }
        if(video_chan_buf_pool->video_packet_entry_pool != NULL){
            isil_video_packet_tcb_t   *ptr_video_packet_tcb;
            int i;
            for(i=0; i<video_chan_buf_pool->video_packet_entry_number; i++) {
                ptr_video_packet_tcb = &video_chan_buf_pool->video_packet_entry_pool[i];
                if(ptr_video_packet_tcb->data != NULL) {
                    free_page((unsigned long)ptr_video_packet_tcb->data);
                    ptr_video_packet_tcb->data = NULL;
                }
            }
            kfree(video_chan_buf_pool->video_packet_entry_pool);
            video_chan_buf_pool->video_packet_entry_pool = NULL;
        }
        video_chan_buf_pool->video_packet_entry_number = 0;
        video_chan_buf_pool->cache_order = 0;

    }
}

static void isil_video_chan_buf_pool_get_video_packet_tcb(isil_video_chan_buf_pool_t *video_chan_buf_pool, isil_video_packet_tcb_t **packet)
{
    if((video_chan_buf_pool!=NULL) && (packet!=NULL)){
        tcb_node_pool_t *video_packet_pool_tcb = &video_chan_buf_pool->video_packet_pool_tcb;
        *packet = NULL;
        if(video_packet_pool_tcb->op != NULL){
            tcb_node_t	*temp_node;
            video_packet_pool_tcb->op->get(video_packet_pool_tcb, &temp_node);
            if(temp_node != NULL){
                *packet = to_video_packet_buf_tcb(temp_node);
                atomic_inc(&((*packet)->ref_count));
            }
        }
    }
}

static void isil_video_chan_buf_pool_try_get_video_packet_tcb(isil_video_chan_buf_pool_t *video_chan_buf_pool, isil_video_packet_tcb_t **packet)
{
    if((video_chan_buf_pool!=NULL) && (packet!=NULL)){
        tcb_node_pool_t *video_packet_pool_tcb = &video_chan_buf_pool->video_packet_pool_tcb;
        *packet = NULL;
        if(video_packet_pool_tcb->op != NULL){
            tcb_node_t	*temp_node;
            video_packet_pool_tcb->op->try_get(video_packet_pool_tcb, &temp_node);
            if(temp_node != NULL){
                *packet = to_video_packet_buf_tcb(temp_node);
                atomic_inc(&((*packet)->ref_count));
            }
        }
    }
}

static void isil_video_chan_buf_pool_put_video_packet_tcb(isil_video_chan_buf_pool_t *video_chan_buf_pool, isil_video_packet_tcb_t *packet)
{
    if((video_chan_buf_pool!=NULL) && (packet!=NULL)){
        tcb_node_pool_t *video_packet_pool_tcb = &video_chan_buf_pool->video_packet_pool_tcb;
        if(video_packet_pool_tcb->op != NULL){
            video_packet_pool_tcb->op->put(video_packet_pool_tcb, &packet->packet_node);
        }
    }
}

static int	isil_video_chan_buf_pool_get_video_packet_tcb_pool_entry_number(isil_video_chan_buf_pool_t *video_chan_buf_pool)
{
    int	entry_number = 0;
    if(video_chan_buf_pool != NULL){
        tcb_node_pool_t *video_packet_pool_tcb = &video_chan_buf_pool->video_packet_pool_tcb;
        if(video_packet_pool_tcb->op != NULL){
            entry_number = video_packet_pool_tcb->op->get_curr_pool_entry_number(video_packet_pool_tcb);
        }
    }
    return entry_number;
}

static void	isil_video_chan_buf_pool_get_video_frame_tcb(isil_video_chan_buf_pool_t *video_chan_buf_pool, isil_video_frame_tcb_t **frame)
{
    if((video_chan_buf_pool!=NULL) && (frame!=NULL)){
        tcb_node_pool_t *video_frame_pool_tcb = &video_chan_buf_pool->video_frame_pool_tcb;
        *frame = NULL;
        if(video_frame_pool_tcb->op != NULL){
            tcb_node_t	*temp_node;
            video_frame_pool_tcb->op->get(video_frame_pool_tcb, &temp_node);
            if(temp_node != NULL){
                *frame = to_video_frame_tcb(temp_node);
                atomic_inc(&((*frame)->ref_count));
            }
        }
    }
}

static void	isil_video_chan_buf_pool_try_get_video_frame_tcb(isil_video_chan_buf_pool_t *video_chan_buf_pool, isil_video_frame_tcb_t **frame)
{
    if((video_chan_buf_pool!=NULL) && (frame!=NULL)){
        tcb_node_pool_t *video_frame_pool_tcb = &video_chan_buf_pool->video_frame_pool_tcb;
        *frame = NULL;
        if(video_frame_pool_tcb->op != NULL){
            tcb_node_t	*temp_node;
            video_frame_pool_tcb->op->try_get(video_frame_pool_tcb, &temp_node);
            if(temp_node != NULL){
                *frame = to_video_frame_tcb(temp_node);
                atomic_inc(&((*frame)->ref_count));
            }
        }
    }
}

static void	isil_video_chan_buf_pool_put_video_frame_tcb(isil_video_chan_buf_pool_t *video_chan_buf_pool, isil_video_frame_tcb_t *frame)
{
    if((video_chan_buf_pool!=NULL) && (frame!=NULL)){
        tcb_node_pool_t *video_frame_pool_tcb = &video_chan_buf_pool->video_frame_pool_tcb;
        if(video_frame_pool_tcb->op != NULL){
            video_frame_pool_tcb->op->put(video_frame_pool_tcb, &frame->frame_node);
        }
    }
}

static int	isil_video_chan_buf_pool_get_video_frame_tcb_pool_entry_number(isil_video_chan_buf_pool_t *video_chan_buf_pool)
{
    int	entry_number = 0;
    if(video_chan_buf_pool != NULL){
        tcb_node_pool_t *video_frame_pool_tcb = &video_chan_buf_pool->video_frame_pool_tcb;
        if(video_frame_pool_tcb->op != NULL){
            entry_number = video_frame_pool_tcb->op->get_curr_pool_entry_number(video_frame_pool_tcb);
        }
    }
    return entry_number;
}

static struct isil_video_chan_buf_pool_operation	isil_video_chan_buf_pool_op = {
    .create = isil_video_chan_buf_pool_create,
    .release = isil_video_chan_buf_pool_release,

    .get_video_packet_tcb = isil_video_chan_buf_pool_get_video_packet_tcb,
    .try_get_video_packet_tcb = isil_video_chan_buf_pool_try_get_video_packet_tcb,
    .put_video_packet_tcb = isil_video_chan_buf_pool_put_video_packet_tcb,
    .get_video_packet_tcb_pool_entry_number = isil_video_chan_buf_pool_get_video_packet_tcb_pool_entry_number,

    .get_video_frame_tcb = isil_video_chan_buf_pool_get_video_frame_tcb,
    .try_get_video_frame_tcb = isil_video_chan_buf_pool_try_get_video_frame_tcb,
    .put_video_frame_tcb = isil_video_chan_buf_pool_put_video_frame_tcb,
    .get_video_frame_tcb_pool_entry_number = isil_video_chan_buf_pool_get_video_frame_tcb_pool_entry_number,
};

int	init_isil_video_chan_buf_pool(isil_video_chan_buf_pool_t *video_chan_buf_pool, unsigned long buf_len)
{
    int	ret = ISIL_ERR;
    if(video_chan_buf_pool != NULL){
        video_chan_buf_pool->op = &isil_video_chan_buf_pool_op;
        ret = video_chan_buf_pool->op->create(video_chan_buf_pool, buf_len);
    }
    return ret;
}

void	remove_isil_video_chan_buf_pool(isil_video_chan_buf_pool_t *video_chan_buf_pool)
{
    if(video_chan_buf_pool != NULL){
        if(video_chan_buf_pool->op != NULL){
            video_chan_buf_pool->op->release(video_chan_buf_pool);
        }
    }
}
